/**************************************************************

phot_head.h (C-Munipack project)
Photometry file header
Copyright (C) 2003 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#ifndef PHOT_HEAD_H
#define PHOT_HEAD_H

#include "hash.h"
#include "context.h"
#include "cmpack_console.h"

/***********************      Datatypes      ******************/

/* Single header item */
struct _CmpackHeadItem
{
	char *key;					/**< Parameter name (case sensitive) */
	char *val;					/**< Parameter value */
	char *com;					/**< Comment text */
};
typedef struct _CmpackHeadItem CmpackHeadItem;
	
/* File header */
struct _CmpackHeader
{
	int	count;						/**< Number of sections */
	int	capacity;					/**< Number of allocated records */
	CmpackHeadItem **list;			/**< List of section items */
	CmpackHashTable hash;			/**< Hash table of items */
};
typedef struct _CmpackHeader CmpackHeader;

/******************   Public functions    *********************/

/* Init file header */
void pht_head_init(CmpackHeader *head);
	
/* Clear header content */
void pht_head_clear(CmpackHeader *head);
	
/* Copy header content */
void pht_head_copy(CmpackHeader *dsthead, const CmpackHeader *srchead);
	
/* Dump content of header to stream */
void pht_head_dump(CmpackConsole *ctx, CmpackHeader *head);
	
/* Write context of header to stream */
void pht_head_write(CmpackHeader *head, FILE *to);
	
/* Insert new parameter or modify existing one */
void pht_head_set(CmpackHeader *head, const char *key, const char *val, const char *com);

/* Get pointer to header item */
CmpackHeadItem *pht_head_get(CmpackHeader *head, const char *key);

/* Delete parameter from a section */
void pht_head_delete(CmpackHeader *head, const char *key);

#endif
