/**************************************************************

aper.c (C-Munipack project)
Table of apertures (photometry file)
Copyright (C) 2003 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "comfun.h"
#include "cmpack_common.h"
#include "cmpack_list.h"
#include "stars.h"

/********************   Public functions    ***********************/

/* Init table */
void list_st_init(CmpackStarList *tab)
{
	tab->list = NULL;
	tab->count = 0;
}

/* Clear table */
void list_st_clear(CmpackStarList *tab)
{
	if (tab->list) {
		cmpack_free(tab->list);
		tab->list = NULL;
	}
	tab->count = 0;
}

/* Set list of stars */
void list_st_set(CmpackStarList *tab, const int *items, int count)
{
	int i;

	if (count>0) {
		tab->count = count;
		tab->list = (CmpackStar*)cmpack_realloc(tab->list, tab->count*sizeof(CmpackStar));
		memset(tab->list, 0, tab->count*sizeof(CmpackStar));
		for (i=0; i<count; i++)
			tab->list[i].id = items[i];
	} else {
		if (tab->list) {
			cmpack_free(tab->list);
			tab->list = NULL;
		}
		tab->count = 0;
	}
}

/* Get list of stars */
void list_st_get(CmpackStarList *tab, int **items, int *nitems)
{
	int i;

	*items = (int*) cmpack_malloc(tab->count*sizeof(int));
	for (i=0; i<tab->count; i++)
		(*items)[i] = tab->list[i].id;
	*nitems = tab->count;
}
