/**************************************************************

head.h (C-Munipack project)
Table header
Copyright (C) 2003 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#ifndef CMPACK_TABLE_HEAD_H
#define CMPACK_TABLE_HEAD_H

#include "cmpack_console.h"

/***********************      Datatypes      ******************/

/* Single header item */
typedef struct _CmpackTabHeadItem CmpackTabHeadItem;
	
/* File header */
typedef struct _CmpackTabHeader CmpackTabHeader;

/******************   Public functions    *********************/

/* Create a new header */
CmpackTabHeader *tab_head_create(void);
	
/* Free allocated memory */
void tab_head_free(CmpackTabHeader *header);
	
/* Dump header content */
void tab_head_dump(CmpackTabHeader *head, CmpackConsole *con);
	
/* Dump header content */
void tab_head_write(CmpackTabHeader *head, FILE *to, const char *prefix);
	
/* Clear header content and clear valid flag */
void tab_head_clear(CmpackTabHeader *head);
	
/* Copy header content and set valid flag */
void tab_head_copy(CmpackTabHeader *dsthead, const CmpackTabHeader *srchead);
		
/* Insert new header item or modify existing one
 * Returns zero on success or error code on failure.
 */
void tab_head_set(CmpackTabHeader *head, const char *key, const char *val);

/* Get number of items */
int tab_head_count(CmpackTabHeader *head);

/* Get parameter by index */
CmpackTabHeadItem *tab_head_get(CmpackTabHeader *head, int index);

/* Search parameter by name */
CmpackTabHeadItem *tab_head_find(CmpackTabHeader *head, const char *key);
	
/* Delete header item */
void tab_head_delete(CmpackTabHeader *head, CmpackTabHeadItem *item);

/* Get parameter by index */
const char *tab_head_item_key(CmpackTabHeadItem *item);

/* Get parameter by index */
const char *tab_head_item_val(CmpackTabHeadItem *item);

#endif
