/**************************************************************

config_dlg.cpp (C-Munipack project)
Configuration dialog
Copyright (C) 2008 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#include <stdlib.h>
#include <string.h>
#include <glib.h>
#include <cmunipack.h>

#include "utils.h"
#include "configuration.h"
#include "jdconv_dlg.h"
#include "main_dlg.h"

//-------------------------   JD CONVERTER DIALOG   --------------------------------

bool	CJDConvDlg::ms_NoInput = true;
double	CJDConvDlg::ms_LastJD = 0;

CJDConvDlg::CJDConvDlg(GtkWindow *pParent)
{
	GtkWidget *tbox, *label;

	m_ActMode = (tMode)CConfig::GetInt("JDConv", "Mode");

	// Dialog with buttons
	m_pDlg = gtk_dialog_new_with_buttons("JD converter", pParent, 
		(GtkDialogFlags)(GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT),
		GTK_STOCK_REFRESH, GTK_RESPONSE_NONE, GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE, NULL);

	// Dialog icon
	gchar *icon = get_icon_file("jdconv");
	gtk_window_set_icon(GTK_WINDOW(m_pDlg), gdk_pixbuf_new_from_file(icon, NULL));
	g_free(icon);

	// Dialog layout
	tbox = gtk_table_new(3, 3, FALSE);
	gtk_container_add (GTK_CONTAINER(GTK_DIALOG(m_pDlg)->vbox), tbox);
	gtk_container_set_border_width(GTK_CONTAINER(tbox), 8);
	gtk_table_set_row_spacings(GTK_TABLE(tbox), 4);
	gtk_table_set_row_spacing(GTK_TABLE(tbox), 0, 16);
	gtk_table_set_row_spacing(GTK_TABLE(tbox), 1, 16);
	gtk_table_set_col_spacings(GTK_TABLE(tbox), 8);

	// Options
	label = gtk_label_new("Compute");
	gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(tbox), label, 0, 1, 0, 1);
	m_Mode = gtk_combo_box_new_text();
	gtk_combo_box_append_text(GTK_COMBO_BOX(m_Mode), "Julian date to date and time");
	gtk_combo_box_append_text(GTK_COMBO_BOX(m_Mode), "Date and time to Julian date");
	gtk_combo_box_set_active(GTK_COMBO_BOX(m_Mode), m_ActMode);
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_Mode, 1, 2, 0, 1);

	// Input entry
	m_Label1 = gtk_label_new("Julian date");
	gtk_misc_set_alignment(GTK_MISC(m_Label1), 1.0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_Label1, 0, 1, 1, 2);
	m_Edit1 = gtk_entry_new_with_max_length(32);
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_Edit1, 1, 2, 1, 2);
	m_Unit1 = gtk_label_new("[24xxxxx.xxx]");
	gtk_misc_set_alignment(GTK_MISC(m_Unit1), 0.0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_Unit1, 2, 3, 1, 2);

	// Output entry
	m_Label2 = gtk_label_new("Date and time");
	gtk_misc_set_alignment(GTK_MISC(m_Label2), 1.0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_Label2, 0, 1, 2, 3);
	m_Edit2 = gtk_entry_new_with_max_length(32);
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_Edit2, 1, 2, 2, 3);
	gtk_editable_set_editable(GTK_EDITABLE(m_Edit2), false);
	m_Unit2 = gtk_label_new("[yyyy-mm-dd hh:mm:ss.sss]");
	gtk_misc_set_alignment(GTK_MISC(m_Unit2), 0.0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_Unit2, 2, 3, 2, 3);

	SetMode(m_ActMode);

	gtk_widget_show_all(tbox);

	// Signals:
	GtkWidget *btn = get_dialog_widget_by_response(m_pDlg, GTK_RESPONSE_NONE);
	g_signal_connect(G_OBJECT(btn), "clicked", G_CALLBACK(button_clicked), this);
	g_signal_connect(G_OBJECT(m_pDlg), "response", G_CALLBACK(response_dialog), this);
	g_signal_connect(G_OBJECT(m_Mode), "changed", G_CALLBACK(select_changed), this);
}

CJDConvDlg::~CJDConvDlg()
{
	gtk_widget_destroy(m_pDlg);
}

void CJDConvDlg::response_dialog(GtkDialog *pDlg, gint response_id, CJDConvDlg *pMe)
{
	if (response_id == GTK_RESPONSE_NONE) 
		g_signal_stop_emission_by_name(pDlg, "response");
}

void CJDConvDlg::select_changed(GtkComboBox *pWidget, CJDConvDlg *pMe)
{
	pMe->SetMode((tMode)gtk_combo_box_get_active(pWidget));
}

void CJDConvDlg::button_clicked(GtkWidget *pButton, CJDConvDlg *pMe)
{
	int response_id;
	
	response_id = gtk_dialog_get_response_for_widget(GTK_DIALOG(pMe->m_pDlg), pButton);
	if (response_id == GTK_RESPONSE_NONE) {
		pMe->GetData();
		pMe->SetData();
	}
}

void CJDConvDlg::Execute()
{
	gtk_dialog_run(GTK_DIALOG(m_pDlg));
	CConfig::Flush();
}

void CJDConvDlg::SetMode(tMode mode)
{
	GetData();

	m_ActMode = mode;
	switch (m_ActMode)
	{
	case MODE_JD_TO_DATETIME:
		gtk_label_set_label(GTK_LABEL(m_Label1), "Julian date");
		gtk_label_set_label(GTK_LABEL(m_Unit1), "[24xxxxx.xxx]");
		gtk_label_set_label(GTK_LABEL(m_Label2), "Date and time");
		gtk_label_set_label(GTK_LABEL(m_Unit2), "[yyyy-mm-dd hh:mm:ss.sss]");
		break;
	case MODE_DATETIME_TO_JD:
		gtk_label_set_label(GTK_LABEL(m_Label1), "Date and time");
		gtk_label_set_label(GTK_LABEL(m_Unit1), "[yyyy-mm-dd hh:mm:ss.sss]");
		gtk_label_set_label(GTK_LABEL(m_Label2), "Julian date");
		gtk_label_set_label(GTK_LABEL(m_Unit2), "[24xxxxx.xxx]");
		break;
	}
	CConfig::SetInt("JDConv", "Mode", m_ActMode);

	SetData();
}

void CJDConvDlg::GetData()
{
	CmpackDateTime dt;
	const char *text;
	char *buf, *ptr;

	ms_LastJD = 0.0;
	text = gtk_entry_get_text(GTK_ENTRY(m_Edit1));
	ms_NoInput = strlen(text)==0;
	if (!ms_NoInput) {
		switch (m_ActMode) 
		{
		case MODE_JD_TO_DATETIME:
			// Read julian date
			buf = g_strdup(text);
			ptr = strchr(buf, ',');
			if (ptr)
				*ptr = '.';
			ms_LastJD = atof(buf);
			g_free(buf);
			break;

		case MODE_DATETIME_TO_JD:
			// Read date and time
			memset(&dt, 0, sizeof(CmpackDateTime));
			if (cmpack_strtodate(text, &dt.date)==0) {
				const char *time = strchr(text, ' ');
				if (time) {
					if (cmpack_strtotime(time+1, &dt.time)==0) 
						ms_LastJD = cmpack_encodejd(&dt);
				} else {
					ms_LastJD = cmpack_encodejd(&dt);
				}
			}
			break;
		}
	}
}

void CJDConvDlg::SetData()
{
	CmpackDateTime dt;
	char buf[256];

	if (ms_NoInput) {
		gtk_entry_set_text(GTK_ENTRY(m_Edit2), "");
	} else {
		switch (m_ActMode) 
		{
		case MODE_JD_TO_DATETIME:
			if (cmpack_decodejd(ms_LastJD, &dt)==0) {
				sprintf(buf, "%.7f", ms_LastJD);
				gtk_entry_set_text(GTK_ENTRY(m_Edit1), buf);
				sprintf(buf, "%04d-%02d-%02d %d:%02d:%02d.%03d", dt.date.year, dt.date.month, dt.date.day,
					dt.time.hour, dt.time.minute, dt.time.second, dt.time.milisecond);
				gtk_entry_set_text(GTK_ENTRY(m_Edit2), buf);
			} else {
				gtk_entry_set_text(GTK_ENTRY(m_Edit2), "Invalid julian date");
			}
			break;

		case MODE_DATETIME_TO_JD:
			if (cmpack_decodejd(ms_LastJD, &dt)==0) {
				sprintf(buf, "%04d-%02d-%02d %d:%02d:%02d.%03d", dt.date.year, dt.date.month, dt.date.day,
					dt.time.hour, dt.time.minute, dt.time.second, dt.time.milisecond);
				gtk_entry_set_text(GTK_ENTRY(m_Edit1), buf);
				sprintf(buf, "%.7f", ms_LastJD);
				gtk_entry_set_text(GTK_ENTRY(m_Edit2), buf);
			} else {
				gtk_entry_set_text(GTK_ENTRY(m_Edit2), "Invalid date or time");
			}		
			break;
		}
	}
}
