/**************************************************************

main_dlg.cpp (C-Munipack project)
Main dialog
Copyright (C) 2008 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#include <stdlib.h>
#include <glib.h>
#include <string.h>
#include <assert.h>

#include "utils.h"
#include "main.h"
#include "configuration.h"
#include "main_dlg.h"
#include "addfiles_dlg.h"
#include "addfolder_dlg.h"
#include "convert_dlg.h"
#include "messages_dlg.h"
#include "project_dlg.h"
#include "timecorr_dlg.h"
#include "biascorr_dlg.h"
#include "darkcorr_dlg.h"
#include "flatcorr_dlg.h"
#include "photometry_dlg.h"
#include "matching_dlg.h"
#include "stars_dlg.h"
#include "aperture_dlg.h"
#include "lightcurve_dlg.h"
#include "chart_dlg.h"
#include "config_dlg.h"
#include "about_dlg.h"
#include "frame_dlg.h"
#include "ccdfile_dlg.h"
#include "open_dlg.h"
#include "masterbias_dlg.h"
#include "masterdark_dlg.h"
#include "masterflat_dlg.h"
#include "varfind_dlg.h"
#include "tracklist_dlg.h"
#include "amasslist_dlg.h"
#include "jdconv_dlg.h"
#include "helcor_dlg.h"
#include "airmass_dlg.h"
#include "newfiles_dlg.h"
#include "phtfile_dlg.h"
#include "catfile_dlg.h"
#include "table_dlg.h"
#include "object_dlg.h"
#include "observer_dlg.h"
#include "readall_dlg.h"
#include "makecat_dlg.h"

enum tMenuId
{
	MENU_FILES = 1,
	MENU_CORRECTIONS,
	MENU_PHOTOMETRY,
	MENU_MATCHING,
	MENU_PLOTTING,
	MENU_TOOLS,
	MENU_WINDOWS,
	MENU_HELP
};

enum tCommandId
{
	CMD_CLEAR_PROJECT = 100,
	CMD_ADD_FILES,
	CMD_ADD_FOLDER,
	CMD_REMOVE_FILES,
	CMD_CONVERT,
	CMD_OPEN_FILE,
	CMD_TIME_CORR,
	CMD_BIAS_CORR,
	CMD_DARK_CORR,
	CMD_FLAT_CORR,
	CMD_PHOTOMETRY,
	CMD_MATCHING,
	CMD_CHOOSE_STARS,
	CMD_CHOOSE_APER,
	CMD_OBJ_COORDS,
	CMD_OBS_COORDS,
	CMD_PLOT_LIGHT_CURVE,
	CMD_PLOT_CHART,
	CMD_FIND_VARIABLES,
	CMD_MASTER_BIAS,
	CMD_MASTER_DARK,
	CMD_MASTER_FLAT,
	CMD_PLOT_TRACK_LIST,
	CMD_PLOT_AIRMASS,
	CMD_NEW_FILES,
	CMD_EXPRESS_REDUCTION,
	CMD_MAKE_CAT_FILE,
	CMD_TOOL_JDCONV,
	CMD_TOOL_HELCORR,
	CMD_TOOL_AIRMASS,
	CMD_PROJECT_INFO,
	CMD_PREFERENCES,
	CMD_EXIT_APP,
	CMD_MESSAGES,
	CMD_ABOUT,
	CMD_OPEN_HELP,
	CMD_SHOW_TABLE,
	CMD_SHOW_ICONS
};

//-------------------------   MAIN TOOLBAR   ---------------------------

static const CToolBar::tToolBtn MainToolBar[] = {
	{ CToolBar::TB_PUSHBUTTON, CMD_CLEAR_PROJECT, "Clear the list of input files", "clearfiles" },
	{ CToolBar::TB_PUSHBUTTON, CMD_ADD_FILES, "Add individual files to the list of input files", "addfiles" },
	{ CToolBar::TB_PUSHBUTTON, CMD_ADD_FOLDER, "Add all files from a folder to the list of input files", "addfolder" },
	{ CToolBar::TB_PUSHBUTTON, CMD_CONVERT, "Convert input files to working format", "convert" },
	{ CToolBar::TB_SEPARATOR },
	{ CToolBar::TB_PUSHBUTTON, CMD_TIME_CORR, "Correct date and time of observation", "timecorr" },
	{ CToolBar::TB_PUSHBUTTON, CMD_BIAS_CORR, "Make bias-frame correction", "biascorr" },
	{ CToolBar::TB_PUSHBUTTON, CMD_DARK_CORR, "Make dark-frame correction", "darkcorr" },
	{ CToolBar::TB_PUSHBUTTON, CMD_FLAT_CORR, "Make flat-frame correction", "flatcorr" },
	{ CToolBar::TB_SEPARATOR },
	{ CToolBar::TB_PUSHBUTTON, CMD_PHOTOMETRY, "Run photometry", "photometry" },
	{ CToolBar::TB_SEPARATOR },
	{ CToolBar::TB_PUSHBUTTON, CMD_MATCHING, "Find cross-references between photometry files", "matchstars" },
	{ CToolBar::TB_SEPARATOR },
	{ CToolBar::TB_PUSHBUTTON, CMD_CHOOSE_STARS, "Select variable, comparison and check stars", "stars" },
	{ CToolBar::TB_PUSHBUTTON, CMD_CHOOSE_APER, "Select aperture", "aperture" },
	{ CToolBar::TB_PUSHBUTTON, CMD_OBJ_COORDS, "Enter object's coordinates", "objcoords" },
	{ CToolBar::TB_PUSHBUTTON, CMD_OBS_COORDS, "Enter observer's coordinates", "obscoords" },
	{ CToolBar::TB_PUSHBUTTON, CMD_PLOT_LIGHT_CURVE, "Plot light-curve graph", "lightcurve" },
	{ CToolBar::TB_PUSHBUTTON, CMD_PLOT_CHART, "Plot chart", "chart" },
	{ CToolBar::TB_SEPARATOR },
	{ CToolBar::TB_PUSHBUTTON, CMD_FIND_VARIABLES, "Find variables", "varfind" },
	{ CToolBar::TB_PUSHBUTTON, CMD_NEW_FILES, "Process new files", "newfiles" },
//	{ CToolBar::TB_PUSHBUTTON, CMD_EXPRESS_REDUCTION, "Express reduction", "express" },
	{ CToolBar::TB_PUSHBUTTON, CMD_PREFERENCES, "Shows the configuration dialog", "preferences" },
	{ CToolBar::TB_SEPARATOR },
	{ CToolBar::TB_PUSHBUTTON, CMD_SHOW_TABLE, "Show the input file in a table", "treeview" },
	{ CToolBar::TB_PUSHBUTTON, CMD_SHOW_ICONS, "Show the input file as icons", "iconview" },
	{ CToolBar::TB_END }
};

//-------------------------   MAIN MENU   ---------------------------

static const CMenuBar::tMenuItem FilesMenu[] = {
	{ CMenuBar::MB_ITEM, CMD_CLEAR_PROJECT, "_Clear files", "clearfiles" },
	{ CMenuBar::MB_ITEM, CMD_ADD_FILES, "_Add files", "addfiles" },
	{ CMenuBar::MB_ITEM, CMD_ADD_FOLDER, "Add _folder", "addfolder" },
	{ CMenuBar::MB_ITEM, CMD_REMOVE_FILES, "_Remove files", "removefiles" },
	{ CMenuBar::MB_ITEM, CMD_CONVERT, "Fetch/convert files", "convert" },
	{ CMenuBar::MB_SEPARATOR },
	{ CMenuBar::MB_ITEM, CMD_OPEN_FILE, "_Open file", "fileopen" },
	{ CMenuBar::MB_SEPARATOR },
	{ CMenuBar::MB_ITEM, CMD_EXIT_APP, "E_xit", "exit" },
	{ CMenuBar::MB_END }
};

static const CMenuBar::tMenuItem CorrMenu[] = {
	{ CMenuBar::MB_ITEM, CMD_TIME_CORR, "_Time correction", "timecorr" },
	{ CMenuBar::MB_ITEM, CMD_BIAS_CORR, "_Bias correction", "biascorr" },
	{ CMenuBar::MB_ITEM, CMD_DARK_CORR, "_Dark correction", "darkcorr" },
	{ CMenuBar::MB_ITEM, CMD_FLAT_CORR, "_Flat correction", "flatcorr" },
	{ CMenuBar::MB_END }
};

static const CMenuBar::tMenuItem PhotMenu[] = {
	{ CMenuBar::MB_ITEM, CMD_PHOTOMETRY, "_Photometry", "photometry" },
	{ CMenuBar::MB_END }
};

static const CMenuBar::tMenuItem MatchMenu[] = {
	{ CMenuBar::MB_ITEM, CMD_MATCHING, "_Match stars", "matchstars" },
	{ CMenuBar::MB_END }
};

static const CMenuBar::tMenuItem PlotMenu[] = {
	{ CMenuBar::MB_ITEM, CMD_CHOOSE_STARS, "Choose _stars", "stars" },
	{ CMenuBar::MB_ITEM, CMD_CHOOSE_APER, "Choose _aperture", "aperture" },
	{ CMenuBar::MB_ITEM, CMD_OBJ_COORDS, "Enter _object's coordinates", "objcoords" },
	{ CMenuBar::MB_ITEM, CMD_OBS_COORDS, "Enter _observer's coordinates", "obscoords" },
	{ CMenuBar::MB_SEPARATOR },
	{ CMenuBar::MB_ITEM, CMD_PLOT_LIGHT_CURVE, "Plot _light-curve", "lightcurve" },
	{ CMenuBar::MB_ITEM, CMD_PLOT_CHART, "Plot _chart", "chart" },
	{ CMenuBar::MB_END }
};

static const CMenuBar::tMenuItem WinMenu[] = {
	{ CMenuBar::MB_ITEM, CMD_MESSAGES, "Show _message log", "messages" },
	{ CMenuBar::MB_ITEM, CMD_PROJECT_INFO, "Show _project info", "project" },
	{ CMenuBar::MB_END }
};

static const CMenuBar::tMenuItem ToolsMenu[] = {
	{ CMenuBar::MB_ITEM, CMD_MASTER_BIAS, "Master _bias-frame", "masterbias" },
	{ CMenuBar::MB_ITEM, CMD_MASTER_DARK, "Master _dark-frame", "masterdark" },
	{ CMenuBar::MB_ITEM, CMD_MASTER_FLAT, "Master _flat-frame", "masterflat" },
	{ CMenuBar::MB_SEPARATOR },
	{ CMenuBar::MB_ITEM, CMD_NEW_FILES, "Process _new files", "newfiles" },
	{ CMenuBar::MB_ITEM, CMD_FIND_VARIABLES, "_Find variables", "varfind" },
	{ CMenuBar::MB_ITEM, CMD_MAKE_CAT_FILE, "_Make catalogue file", "makecat" },
	{ CMenuBar::MB_ITEM, CMD_PLOT_TRACK_LIST, "Plot _track-list", "tracklist" },
	{ CMenuBar::MB_ITEM, CMD_PLOT_AIRMASS, "Plot _air-mass coefficients" },
	{ CMenuBar::MB_SEPARATOR },
	{ CMenuBar::MB_ITEM, CMD_TOOL_JDCONV, "_JD converter", "jdconv" },
	{ CMenuBar::MB_ITEM, CMD_TOOL_HELCORR, "_Heliocentric correction", "helcor" },
	{ CMenuBar::MB_ITEM, CMD_TOOL_AIRMASS, "_Air-mass coefficient", "airmass" },
//	{ CMenuBar::MB_ITEM, CMD_EXPRESS_REDUCTION, "Express reduction", "express" },
	{ CMenuBar::MB_SEPARATOR },
	{ CMenuBar::MB_ITEM, CMD_PREFERENCES, "_Edit preferences", "preferences" },
	{ CMenuBar::MB_END }
};

static const CMenuBar::tMenuItem HelpMenu[] = {
	{ CMenuBar::MB_ITEM, CMD_OPEN_HELP, "_Contents", "help" },
	{ CMenuBar::MB_SEPARATOR },
	{ CMenuBar::MB_ITEM, CMD_ABOUT, "_About MuniWin", "about" },
	{ CMenuBar::MB_END }
};

static const CMenuBar::tMenu MainMenu[] = {
	{ "_Files", MENU_FILES, FilesMenu },
	{ "_Calibration", MENU_CORRECTIONS, CorrMenu },
	{ "_Photometry", MENU_PHOTOMETRY, PhotMenu },
	{ "_Matching", MENU_MATCHING, MatchMenu },
	{ "Plo_tting", MENU_PLOTTING, PlotMenu },
	{ "_Tools", MENU_TOOLS, ToolsMenu },
	{ "_Windows", MENU_WINDOWS, WinMenu },
	{ "_Help", MENU_HELP, HelpMenu, CMenuBar::MBF_RIGHT },
	{ NULL }
};

//-------------------------   HELPER FUNCTIONS   ------------------------------------

static void foreach_selected(GtkTreeModel *model, GtkTreePath *path, 
							   GtkTreeIter *iter, char **fpath)
{
	if (*fpath==NULL) 
		gtk_tree_model_get(model, iter, FRAME_ORIGFILE, fpath, -1);
}

//-------------------------   FILE LIST ------------------------------------

// Icon identifiers
enum tIcon {
	ICO_ERROR,
	ICO_NEW,
	ICO_FETCHED,
	ICO_BIASCORR,
	ICO_TIMECORR,
	ICO_DARKCORR,
	ICO_FLATCORR,
	ICO_PHOTOMETRED,
	ICO_MATCHED
};

// Icon files
static const struct tTreeIcon {
	int id;
	const char *name;		// Icon name
} TreeViewIcons[] = {
	{ ICO_ERROR, "error" },
	{ ICO_NEW, "inputfiles" },
	{ ICO_FETCHED, "convert" },
	{ ICO_BIASCORR, "biascorr" },
	{ ICO_TIMECORR, "timecorr" },
	{ ICO_DARKCORR, "darkcorr" },
	{ ICO_FLATCORR, "flatcorr" },
	{ ICO_PHOTOMETRED, "photometry" },
	{ ICO_MATCHED, "matchstars" },
	{ 0, NULL }
};

static void GetDateTime(GtkTreeViewColumn *tree_column, GtkCellRenderer *cell,
	GtkTreeModel *tree_model, GtkTreeIter *iter, gpointer data)
{
	int state;
	double jd;
	CmpackDateTime dt;
	char buf[256];

	gtk_tree_model_get(tree_model, iter, FRAME_STATE, &state, 
		FRAME_JULDAT, &jd, -1);

	if ((state & CFILE_CONVERSION)!=0) {
		cmpack_decodejd(jd, &dt);
		sprintf(buf, "%04d-%02d-%02d %02d:%02d:%02d.%03d", dt.date.year, dt.date.month, dt.date.day,
			dt.time.hour, dt.time.minute, dt.time.second, dt.time.milisecond);
		g_object_set(cell, "text", buf, NULL);
	} else {
		g_object_set(cell, "text", "", NULL);
	}
}

static void GetJD(GtkTreeViewColumn *tree_column, GtkCellRenderer *cell,
	GtkTreeModel *tree_model, GtkTreeIter *iter, gpointer data)
{
	int state;
	double jd;
	char buf[64];

	gtk_tree_model_get(tree_model, iter, FRAME_STATE, &state, 
		FRAME_JULDAT, &jd, -1);

	if ((state & CFILE_CONVERSION)!=0) {
		sprintf(buf, "%.6f", jd);
		g_object_set(cell, "text", buf, NULL);
	} else {
		g_object_set(cell, "text", "", NULL);
	}
}

static void GetExpTime(GtkTreeViewColumn *tree_column, GtkCellRenderer *cell,
	GtkTreeModel *tree_model, GtkTreeIter *iter, gpointer data)
{
	int state;
	double exp_time;
	char buf[64];

	gtk_tree_model_get(tree_model, iter, FRAME_STATE, &state, 
		FRAME_EXPTIME, &exp_time, -1);

	if ((state & CFILE_CONVERSION)!=0) {
		sprintf(buf, "%.2f", exp_time);
		g_object_set(cell, "text", buf, NULL);
	} else {
		g_object_set(cell, "text", "", NULL);
	}
}

static void GetCCDTemp(GtkTreeViewColumn *tree_column, GtkCellRenderer *cell,
	GtkTreeModel *tree_model, GtkTreeIter *iter, gpointer data)
{
	int state;
	double ccd_temp;
	char buf[64];

	gtk_tree_model_get(tree_model, iter, FRAME_STATE, &state, 
		FRAME_CCDTEMP, &ccd_temp, -1);

	if ((state & CFILE_CONVERSION)!=0) {
		sprintf(buf, "%.1f", ccd_temp);
		g_object_set(cell, "text", buf, NULL);
	} else {
		g_object_set(cell, "text", "", NULL);
	}
}

static void GetStars(GtkTreeViewColumn *tree_column, GtkCellRenderer *cell,
	GtkTreeModel *tree_model, GtkTreeIter *iter, gpointer data)
{
	int state;
	int mstars, nstars;
	char buf[128];

	gtk_tree_model_get(tree_model, iter, FRAME_STATE, &state, 
		FRAME_MSTARS, &mstars, FRAME_STARS, &nstars, -1);

	if ((state & CFILE_MATCHING)!=0) {
		sprintf(buf, "%d/%d", nstars, mstars);
		g_object_set(cell, "text", buf, NULL);
	} else
	if ((state & CFILE_PHOTOMETRY)!=0) {
		sprintf(buf, "%d", nstars);
		g_object_set(cell, "text", buf, NULL);
	} else {
		g_object_set(cell, "text", "", NULL);
	}
}

static void GetIcon(GtkTreeViewColumn *tree_column, GtkCellRenderer *cell,
	GtkTreeModel *tree_model, GtkTreeIter *iter, gpointer data)
{
	int state;
	CMainWindow *pMe = (CMainWindow*)data;

	gtk_tree_model_get(tree_model, iter, FRAME_STATE, &state, -1);

	if ((state & CFILE_ERROR)!=0)
		g_object_set(cell, "pixbuf", pMe->GetIconRef(ICO_ERROR), NULL);
	else if ((state & CFILE_MATCHING)!=0)
		g_object_set(cell, "pixbuf", pMe->GetIconRef(ICO_MATCHED), NULL);
	else if ((state & CFILE_PHOTOMETRY)!=0)
		g_object_set(cell, "pixbuf", pMe->GetIconRef(ICO_PHOTOMETRED), NULL);
	else if ((state & CFILE_FLATCORR)!=0)
		g_object_set(cell, "pixbuf", pMe->GetIconRef(ICO_FLATCORR), NULL);
	else if ((state & CFILE_DARKCORR)!=0)
		g_object_set(cell, "pixbuf", pMe->GetIconRef(ICO_DARKCORR), NULL);
	else if ((state & CFILE_TIMECORR)!=0)
		g_object_set(cell, "pixbuf", pMe->GetIconRef(ICO_TIMECORR), NULL);
	else if ((state & CFILE_BIASCORR)!=0)
		g_object_set(cell, "pixbuf", pMe->GetIconRef(ICO_BIASCORR), NULL);
	else if ((state & CFILE_CONVERSION)!=0)
		g_object_set(cell, "pixbuf", pMe->GetIconRef(ICO_FETCHED), NULL);
	else
		g_object_set(cell, "pixbuf", pMe->GetIconRef(ICO_NEW), NULL);
}

const static struct tTreeColumn {
	const char *caption;		// Column name
	int column;					// Model column index
	GtkTreeCellDataFunc datafn;	// Data function
	gfloat align;				// Text alignment
	int bitmask;				// Configuration bit (0 = fixed column)
} TreeViewColumns[] = {
	{ "Frame #",		FRAME_STRINGID,	NULL,			0, 0 },
	{ "Date and time",	0,				GetDateTime,	1, (1<<0) },
	{ "Julian date",	0,				GetJD,			1, (1<<1) },
	{ "Exposure",		0,				GetExpTime,		1, (1<<2) },
	{ "Temperature",	0,				GetCCDTemp,		1, (1<<3) },
	{ "Filter",			FRAME_FILTER,	NULL,			0, (1<<4) },
	{ "Stars",			0,				GetStars,		1, (1<<5) },
	{ "Original file",  FRAME_ORIGFILE,	NULL,			0, (1<<6) },
	{ "Temporary file", FRAME_TEMPFILE,	NULL,			0, (1<<7) },
	{ "Status",			FRAME_REPORT,	NULL,			0, 0 },
	{ NULL }
};

//-------------------------   MAIN WINDOW   --------------------------------

CMainWindow::CMainWindow():m_pDlg(NULL), m_Previews(NULL), m_Files(NULL), m_LockCounter(0)
{
	char *icon;
	int i, width, height, count;
	static const GdkGeometry geom = { 480, 240 };
	
	// Configuration
	m_ShowIcons = CConfig::GetBool("Environment", "IconView");

	// Load icons
	count = 0;
	for (i=0; TreeViewIcons[i].name!=NULL; i++)
		count++;
	if (count>0) {
		m_Icons = (GdkPixbuf**)g_malloc(count*sizeof(GdkPixbuf*));
		memset(m_Icons, 0, count*sizeof(GdkPixbuf*));
		for (i=0; i<count; i++) {
			char *icon = get_icon_file(TreeViewIcons[i].name);
			m_Icons[i] = gdk_pixbuf_new_from_file(icon, NULL);
			g_free(icon);
		}
	}

	// Main window
	m_pDlg = (GtkWindow*)gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title(GTK_WINDOW (m_pDlg), "MuniWin");
	gtk_window_set_geometry_hints(GTK_WINDOW(m_pDlg), NULL, (GdkGeometry*)&geom, GDK_HINT_MIN_SIZE);
#ifdef _WIN32
	// Restore window position on Windows systems
	int left, top;
	if (CConfig::TryGetInt("MainDlg", "Left", &left) && CConfig::TryGetInt("MainDlg", "Top", &top)) {
		gtk_window_set_position(GTK_WINDOW(m_pDlg), GTK_WIN_POS_NONE);
		gtk_window_move(GTK_WINDOW(m_pDlg), left, top);
	} else {
		gtk_window_set_position(GTK_WINDOW(m_pDlg), GTK_WIN_POS_CENTER);
	}
#else
	// Let window manager do its work on other systems
	gtk_window_set_position(GTK_WINDOW(m_pDlg), GTK_WIN_POS_CENTER);
#endif
	// Restore window size
	width = CConfig::GetInt("MainDlg", "Width", 640);
	height = CConfig::GetInt("MainDlg", "Height", 320);
	gtk_window_set_default_size(GTK_WINDOW(m_pDlg), width, height);
	// Set window icon
	icon = get_icon_file("muniwin");
	gtk_window_set_icon(GTK_WINDOW (m_pDlg), gdk_pixbuf_new_from_file(icon, NULL));
	g_free(icon);
	g_signal_connect(G_OBJECT(m_pDlg), "delete_event", G_CALLBACK (exit_event), this);
	gtk_widget_realize(GTK_WIDGET(m_pDlg));
	// Window layout
	m_Box = gtk_vbox_new(FALSE, 0);
	gtk_container_add(GTK_CONTAINER(m_pDlg), m_Box);

	// Menu bar
	m_Menu.Create(MainMenu);
	m_Menu.RegisterCallback(MenuCallback, this);
	gtk_box_pack_start (GTK_BOX(m_Box), m_Menu.Handle(), FALSE, FALSE, 0);
	gtk_widget_show(m_Menu.Handle());

	// Tool bar
	m_TBar.Create(MainToolBar);
	m_TBar.RegisterCallback(ToolbarCallback, this);
	gtk_box_pack_start (GTK_BOX(m_Box), m_TBar.Handle(), FALSE, FALSE, 0);
	gtk_widget_show(m_TBar.Handle());

	// Make tree view
	m_TreeView = gtk_tree_view_new();
	gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(m_TreeView), true);
	g_signal_connect(G_OBJECT(m_TreeView), "row-activated", G_CALLBACK(row_activated), this);
	g_signal_connect(G_OBJECT(m_TreeView), "button_press_event", G_CALLBACK(button_press_event), this);
	gtk_widget_show(m_TreeView);

	// Make scrolled window for a tree view
	m_TreeScrWnd = gtk_scrolled_window_new (NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(m_TreeScrWnd),
		GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(m_TreeScrWnd), 
		GTK_SHADOW_ETCHED_IN);
	gtk_container_add(GTK_CONTAINER(m_TreeScrWnd), m_TreeView);
	gtk_box_pack_start(GTK_BOX(m_Box), m_TreeScrWnd, TRUE, TRUE, 0);
	if (!m_ShowIcons)
		gtk_widget_show_all(m_TreeScrWnd);

	// Make icon view
	m_IconView = gtk_icon_view_new();
	g_signal_connect(G_OBJECT(m_IconView), "item-activated", G_CALLBACK(item_activated), this);
	gtk_widget_show(m_IconView);

	// Make scrolled window for the icon view
	m_IconScrWnd = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(m_IconScrWnd),
		GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(m_IconScrWnd), 
		GTK_SHADOW_ETCHED_IN);
	gtk_container_add(GTK_CONTAINER(m_IconScrWnd), m_IconView);
	gtk_box_pack_start(GTK_BOX(m_Box), m_IconScrWnd, TRUE, TRUE, 0);
	if (m_ShowIcons)
		gtk_widget_show_all(m_IconScrWnd);

	// Status bar
	m_SBar.Create();
	gtk_box_pack_start (GTK_BOX(m_Box), m_SBar.Handle(), FALSE, FALSE, 0);
	gtk_widget_show(m_SBar.Handle());

	// Register callback for selection change
	GtkTreeSelection *selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(m_TreeView));
	gtk_tree_selection_set_mode(selection, GTK_SELECTION_MULTIPLE);
	g_signal_connect(G_OBJECT(selection), "changed", G_CALLBACK(selection_changed), this);

	// Load project file
	Global.pProject->Load();

	RebuildTable();
	UpdateControls();
	gtk_widget_show(m_Box);
	gtk_widget_show(GTK_WIDGET(m_pDlg));
}

CMainWindow::~CMainWindow()
{
	g_free(m_Icons);
	g_slist_free(m_Previews);
	g_slist_free(m_Files);
}

void CMainWindow::RebuildTable()
{
	bool first = true;
	int i, bitmask = CConfig::GetInt("Environment", "Columns", 53);
	GList *list, *ptr;
	const tTreeColumn *col;
	GtkTreeViewColumn *tvcol;
	GtkCellRenderer *renderer;

	// Rebuild tree view
	gtk_tree_view_set_model(GTK_TREE_VIEW(m_TreeView), NULL);
	// Delete old columns
	list = gtk_tree_view_get_columns(GTK_TREE_VIEW(m_TreeView));
	for (ptr=list; ptr!=NULL; ptr=ptr->next) {
		tvcol = (GtkTreeViewColumn*)(ptr->data);
		gtk_tree_view_remove_column(GTK_TREE_VIEW(m_TreeView), tvcol);
	}
	g_list_free(list);
	// Create new columns
	for (i=0; TreeViewColumns[i].caption!=NULL; i++) {
		col = &TreeViewColumns[i];
		if (col->bitmask==0 || (col->bitmask & bitmask)!=0) {
			tvcol = gtk_tree_view_column_new();
			// Set column name and alignment
			gtk_tree_view_column_set_title(tvcol, col->caption);
			gtk_tree_view_append_column(GTK_TREE_VIEW(m_TreeView), tvcol);
			// Add pixbuf renderer to the first column
			if (first) {
				renderer = gtk_cell_renderer_pixbuf_new();
				gtk_tree_view_column_pack_start(tvcol, renderer, FALSE);
				gtk_tree_view_column_set_cell_data_func(tvcol, renderer, GetIcon, this, NULL);
				first = false;
			}
			// Add text renderer
			renderer = gtk_cell_renderer_text_new();
			gtk_tree_view_column_pack_start(tvcol, renderer, TRUE);
			g_object_set(renderer, "xalign", col->align, NULL);
			if (col->datafn) 
				gtk_tree_view_column_set_cell_data_func(tvcol, renderer, col->datafn, NULL, NULL);
			else
				gtk_tree_view_column_add_attribute(tvcol, renderer, "text", col->column);
		}
	}
	gtk_tree_view_set_model(GTK_TREE_VIEW(m_TreeView), Global.pProject->FileList());

	// Rebuild icon view
	gtk_icon_view_set_model(GTK_ICON_VIEW(m_IconView), NULL);
	gtk_icon_view_set_text_column(GTK_ICON_VIEW(m_IconView), FRAME_STRINGID);
	gtk_icon_view_set_pixbuf_column(GTK_ICON_VIEW(m_IconView), FRAME_ICON);
	gtk_icon_view_set_model(GTK_ICON_VIEW(m_IconView), Global.pProject->FileList());
}

gint CMainWindow::exit_event(GtkWidget *widget, GdkEvent *event, CMainWindow *pMe)
{
	int width, height;
	
	// Save window position and size
	gtk_window_get_size(GTK_WINDOW(pMe->m_pDlg), &width, &height);
	CConfig::SetInt("MainDlg", "Width", width);
	CConfig::SetInt("MainDlg", "Height", height);
#ifdef _WIN32
	int left, top;
	gtk_window_get_position(GTK_WINDOW(pMe->m_pDlg), &left, &top);
	CConfig::SetInt("MainDlg", "Left", left);
	CConfig::SetInt("MainDlg", "Top", top);
#endif

	// Save project file
	Global.pProject->Save();

	gtk_main_quit();
	return FALSE;
}

void CMainWindow::selection_changed(GtkTreeSelection *widget, CMainWindow *pMe)
{
	pMe->UpdateControls();
}

void CMainWindow::row_activated(GtkTreeView *tree_view, GtkTreePath *path, 
		GtkTreeViewColumn *column, CMainWindow *pMe)
{
	pMe->OnRowActivated(tree_view, path);
}

void CMainWindow::OnRowActivated(GtkTreeView *tree_view, GtkTreePath *path)
{
	ShowFrame(path);
}

void CMainWindow::item_activated(GtkIconView *icon_view, GtkTreePath *path, CMainWindow *pMe)
{
	pMe->OnItemActivated(icon_view, path);
}

void CMainWindow::OnItemActivated(GtkIconView *icon_view, GtkTreePath *path)
{
	ShowFrame(path);
}

void CMainWindow::ShowFrame(GtkTreePath *path)
{
	CFrameDlg *pDlg = FindFrame(path);
	if (!pDlg) {
		pDlg = new CFrameDlg(m_pDlg);
		m_Previews = g_slist_prepend(m_Previews, pDlg);
	}
	pDlg->Show(path);
}

CFrameDlg *CMainWindow::FindFrame(GtkTreePath *pPath)
{
	CFrameDlg *retval = NULL;
	for (GSList *ptr=m_Previews; ptr!=NULL; ptr=ptr->next) {
		GtkTreePath *pPath2 = ((CFrameDlg*)ptr->data)->FramePath();
		if (gtk_tree_path_compare(pPath, pPath2)==0) {
			retval = (CFrameDlg*)ptr->data;
			gtk_tree_path_free(pPath2);
			break;
		} else {
			gtk_tree_path_free(pPath2);
		}
	}
	return retval;
}

void CMainWindow::FrameClosed(CFrameDlg *pDlg)
{
	if (pDlg) 
		m_Previews = g_slist_remove(m_Previews, pDlg);
}

CFileDlg *CMainWindow::FindFile(const char *path)
{
	for (GSList *ptr=m_Files; ptr!=NULL; ptr=ptr->next) {
		const char *path2 = ((CFileDlg*)ptr->data)->Path();
		if (path2) {
			if (strcmp(path2, path)==0)
				return ((CFileDlg*)ptr->data);
		}
	}
	return NULL;
}

void CMainWindow::FileClosed(CFileDlg *pDlg)
{
	if (pDlg)
		m_Files = g_slist_remove(m_Files, pDlg);
}

void CMainWindow::OnCommand(int cmd_id)
{
	switch (cmd_id)
	{
	case CMD_ADD_FILES:
		AddFiles();
		break;
	case CMD_ADD_FOLDER:
		AddFolder();
		break;
	case CMD_CLEAR_PROJECT:
		ClearProject();
		break;
	case CMD_REMOVE_FILES:
		RemoveFiles();
		break;
	case CMD_CONVERT:
		ConvertFiles();
		break;
	case CMD_OPEN_FILE:
		OpenFile();
		break;
	case CMD_EXIT_APP:
		ExitApplication();
		break;

	case CMD_TIME_CORR:
		TimeCorrection();
		break;
	case CMD_BIAS_CORR:
		BiasCorrection();
		break;
	case CMD_DARK_CORR:
		DarkCorrection();
		break;
	case CMD_FLAT_CORR:
		FlatCorrection();
		break;

	case CMD_PHOTOMETRY:
		Photometry();
		break;

	case CMD_MATCHING:
		Matching();
		break;

	case CMD_CHOOSE_STARS:
		ChooseStars();
		break;
	case CMD_CHOOSE_APER:
		ChooseAperture();
		break;
	case CMD_OBJ_COORDS:
		EnterObjectCoords();
		break;
	case CMD_OBS_COORDS:
		EnterObserverCoords();
		break;
	case CMD_PLOT_LIGHT_CURVE:
		PlotLightCurve();
		break;
	case CMD_PLOT_CHART:
		PlotChart();
		break;

	case CMD_FIND_VARIABLES:
		FindVariables();
		break;
	case CMD_MAKE_CAT_FILE:
		MakeCatalogFile();
		break;
	case CMD_MASTER_BIAS:
		MasterBias();
		break;
	case CMD_MASTER_DARK:
		MasterDark();
		break;
	case CMD_MASTER_FLAT:
		MasterFlat();
		break;
	case CMD_PLOT_TRACK_LIST:
		PlotTrackList();
		break;
	case CMD_PLOT_AIRMASS:
		PlotAirMass();
		break;
	case CMD_EXPRESS_REDUCTION:
		ExpressReduction();
		break;
	case CMD_NEW_FILES:
		NewFiles();
		break;
	case CMD_TOOL_JDCONV:
		JDConverter();
		break;
	case CMD_TOOL_HELCORR:
		HelCorrection();
		break;
	case CMD_TOOL_AIRMASS:
		AirMass();
		break;
	case CMD_PREFERENCES:
		EditPreferences();
		break;

	case CMD_MESSAGES:
		ShowMessages();
		break;
	case CMD_PROJECT_INFO:
		ShowProject();
		break;

	case CMD_ABOUT:
		AboutApplication();
		break;
	case CMD_OPEN_HELP:
		ShowHelp(m_pDlg);
		break;

	case CMD_SHOW_TABLE:
		ShowTable();
		break;
	case CMD_SHOW_ICONS:
		ShowIcons();
		break;
	}
}

void CMainWindow::MenuCallback(int message, int wparam, 
		void* lparam, void* cb_data)
{
	CMainWindow *pMe = (CMainWindow*)cb_data;

	switch(message)
	{
	case CMenuBar::CB_ACTIVATE:
		// Menu bar command
		pMe->OnCommand(wparam);
		break;
	}
}

void CMainWindow::ToolbarCallback(int message, int wparam, 
		void* lparam, void* cb_data)
{
	CMainWindow *pMe = (CMainWindow*)cb_data;

	switch(message)
	{
	case CToolBar::CB_ACTIVATE:
		// Menu bar command
		pMe->OnCommand(wparam);
		break;
	}
}

// Exit application
void CMainWindow::ExitApplication(void)
{
	gtk_main_quit();
}

// Add files to project
void CMainWindow::AddFiles(void)
{
	CAddFilesDlg pDlg(GTK_WINDOW(m_pDlg));
	pDlg.Execute();
	UpdateControls();
}

// Add folder to project
void CMainWindow::AddFolder(void)
{
	CAddFolderDlg pDlg(GTK_WINDOW(m_pDlg));
	pDlg.Execute();
	UpdateControls();
}

// Clear project
void CMainWindow::ClearProject(void)
{
	if (ShowConfirmation(m_pDlg, "Do you want to remove all files from the current project?")) {
		Global.pProject->Clear();
		UpdateControls();
	}
}

void CMainWindow::RemoveFiles(void)
{
	GtkTreeSelection *pSel = gtk_tree_view_get_selection(GTK_TREE_VIEW(m_TreeView));
	if (gtk_tree_selection_count_selected_rows(pSel)>0) {
		if (ShowConfirmation(m_pDlg, "Do you want to remove selected file(s) from the current project?")) {
			Global.pProject->RemoveFiles(pSel);
			UpdateControls();
		}
	} else {
		ShowError(m_pDlg, "There are no selected files.");
	}
}

// Fetch/convert files
void CMainWindow::ConvertFiles(void)
{
	CConvertDlg pDlg(GTK_WINDOW(m_pDlg));
	pDlg.Execute();
	UpdateControls();
}

// Open file
void CMainWindow::OpenFile(void)
{
	COpenDlg pOpenDlg(GTK_WINDOW(m_pDlg));
	pOpenDlg.Execute();
	UpdateControls();
}

void CMainWindow::ShowFile(const char *fpath)
{
	CFileDlg *pFileDlg = FindFile(fpath);
	if (pFileDlg) {
		// Bring window to front
		pFileDlg->Show();
	} else {
		// Autodetect file type
		switch (FileType(fpath))
		{
		case FILE_TYPE_IMAGE:
			pFileDlg = new CCCDFileDlg(m_pDlg);
			break;
		case FILE_TYPE_PHOT:
			pFileDlg = new CPhtFileDlg(m_pDlg);
			break;
		case FILE_TYPE_CAT:
			pFileDlg = new CCatFileDlg(m_pDlg);
			break;
		case FILE_TYPE_TABLE:
			pFileDlg = new CTableDlg(m_pDlg);
			break;
		case FILE_TYPE_READALL:
			pFileDlg = new CReadAllDlg(m_pDlg);
			break;
		default:
			pFileDlg = NULL;
		}
		if (pFileDlg) {
			// Load file and display it
			if (!pFileDlg->Load(fpath)) {
				delete pFileDlg;
				pFileDlg = NULL;
			}
		}
		if (pFileDlg) {
			m_Files = g_slist_prepend(m_Files, pFileDlg);
			pFileDlg->Show();
		} else {
			ShowError(GTK_WINDOW(m_pDlg), "Unsupported format of file.");
		}
	}
}

// Show message log
void CMainWindow::ShowMessages(void)
{
	CMessagesDlg pDlg(GTK_WINDOW(m_pDlg));
	pDlg.ShowModal();
}

// Show project info
void CMainWindow::ShowProject(void)
{
	CProjectDlg pDlg(GTK_WINDOW(m_pDlg));
	pDlg.ShowModal();
}

// Time correction
void CMainWindow::TimeCorrection(void)
{
	CTimeCorrDlg pDlg(GTK_WINDOW(m_pDlg));
	pDlg.Execute();
	UpdateControls();
}

// Bias correction
void CMainWindow::BiasCorrection(void)
{
	CBiasCorrDlg pDlg(GTK_WINDOW(m_pDlg));
	pDlg.Execute();
	UpdateControls();
}

// Dark correction
void CMainWindow::DarkCorrection(void)
{
	CDarkCorrDlg pDlg(GTK_WINDOW(m_pDlg));
	pDlg.Execute();
	UpdateControls();
}

// Flat correction
void CMainWindow::FlatCorrection(void)
{
	CFlatCorrDlg pDlg(GTK_WINDOW(m_pDlg));
	pDlg.Execute();
	UpdateControls();
}

// Photometry
void CMainWindow::Photometry(void)
{
	CPhotometryDlg pDlg(GTK_WINDOW(m_pDlg));
	pDlg.Execute();
	UpdateControls();
}

// Matching
void CMainWindow::Matching(void)
{
	CMatchingDlg pDlg(GTK_WINDOW(m_pDlg));
	pDlg.Execute();
	UpdateControls();
}

// Choose stars
void CMainWindow::ChooseStars(void)
{
	CStarsDlg pDlg(GTK_WINDOW(m_pDlg));
	pDlg.Execute();
	UpdateControls();
}

// Choose aperture
void CMainWindow::ChooseAperture(void)
{
	CApertureDlg pDlg(GTK_WINDOW(m_pDlg));
	pDlg.Execute();
	UpdateControls();
}

// Enter object's coordinates
void CMainWindow::EnterObjectCoords(void)
{
	CObjectDlg pDlg(GTK_WINDOW(m_pDlg));
	pDlg.Execute();
	UpdateControls();
}

// Enter observer's coordinates
void CMainWindow::EnterObserverCoords(void)
{
	CObserverDlg pDlg(GTK_WINDOW(m_pDlg));
	pDlg.Execute();
	UpdateControls();
}

// Plot light curve
void CMainWindow::PlotLightCurve(void)
{
	CLightCurveDlg pDlg(GTK_WINDOW(m_pDlg));
	pDlg.Execute();
	UpdateControls();
}

// Plot chart
void CMainWindow::PlotChart(void)
{
	CChartDlg pDlg(GTK_WINDOW(m_pDlg));
	pDlg.Execute();
	UpdateControls();
}

// About application
void CMainWindow::AboutApplication(void)
{
	CAboutDlg pDlg(GTK_WINDOW(m_pDlg));
	pDlg.Execute();
	UpdateControls();
}

// Find variables
void CMainWindow::FindVariables(void)
{
	CVarFindDlg pDlg(GTK_WINDOW(m_pDlg));
	pDlg.Execute();
	UpdateControls();
}

// Make catalog file
void CMainWindow::MakeCatalogFile(void)
{
	CMakeCatFileDlg pDlg(GTK_WINDOW(m_pDlg));
	pDlg.Execute();
	UpdateControls();
}

// Master bias
void CMainWindow::MasterBias(void)
{
	CMasterBiasDlg pDlg(GTK_WINDOW(m_pDlg));
	pDlg.Execute();
	UpdateControls();
}

// Master dark 
void CMainWindow::MasterDark(void)
{
	CMasterDarkDlg pDlg(GTK_WINDOW(m_pDlg));
	pDlg.Execute();
	UpdateControls();
}

// Master flat
void CMainWindow::MasterFlat(void)
{
	CMasterFlatDlg pDlg(GTK_WINDOW(m_pDlg));
	pDlg.Execute();
	UpdateControls();
}

// Plot track list
void CMainWindow::PlotTrackList(void)
{
	CTrackListDlg pDlg(GTK_WINDOW(m_pDlg));
	pDlg.Execute();
	UpdateControls();
}

// Plot air-mass coefficients
void CMainWindow::PlotAirMass(void)
{
	CAirMassListDlg pDlg(GTK_WINDOW(m_pDlg));
	pDlg.Execute();
	UpdateControls();
}

// Express reduction
void CMainWindow::ExpressReduction(void)
{
	UpdateControls();
}

// Process new files
void CMainWindow::NewFiles(void)
{
	CNewFilesDlg pDlg(GTK_WINDOW(m_pDlg));
	pDlg.Execute();
	UpdateControls();
}

// JD converter
void CMainWindow::JDConverter(void)
{
	CJDConvDlg pDlg(GTK_WINDOW(m_pDlg));
	pDlg.Execute();
	UpdateControls();
}

// Heliocentric correction
void CMainWindow::HelCorrection(void)
{
	CHelCorDlg pDlg(GTK_WINDOW(m_pDlg));
	pDlg.Execute();
	UpdateControls();
}

// Air-mass computation
void CMainWindow::AirMass(void)
{
	CAirMassDlg pDlg(GTK_WINDOW(m_pDlg));
	pDlg.Execute();
	UpdateControls();
}

// Edit preferences
void CMainWindow::EditPreferences(void)
{
	CConfigDlg pDlg(GTK_WINDOW(m_pDlg));
	pDlg.Execute();
	RebuildTable();
	UpdateControls();
}

void CMainWindow::UpdateControls()
{
	int selected;
	bool advcalib, objcoords, obscoords;
	char *fpath, buf[256];
	CProject::tStatus status;

	// Number of selected items
	GtkTreeSelection *pSel = gtk_tree_view_get_selection(GTK_TREE_VIEW(m_TreeView));
	selected = gtk_tree_selection_count_selected_rows(pSel);

	// Number of m_Files in various states
	Global.pProject->GetStatus(&status);

	// Configuration
	advcalib = CConfig::GetBool("Calibration", "Advanced");
	objcoords = (CConfig::GetInt("LightCurve", "JDFormat")==1) ||
		(CConfig::GetInt("LightCurve", "OptColumns") & 0x01) ||
		(CConfig::GetInt("LightCurve", "OptColumns") & 0x02) ||
		(CConfig::GetInt("LightCurve", "OptColumns") & 0x04);
	obscoords = (CConfig::GetInt("LightCurve", "OptColumns") & 0x02) ||
		(CConfig::GetInt("LightCurve", "OptColumns") & 0x04);

	// Update menu items
	m_Menu.Show(CMD_BIAS_CORR, advcalib);
	m_Menu.Show(CMD_MASTER_BIAS, advcalib);
	m_Menu.Show(CMD_OBJ_COORDS, objcoords);
	m_Menu.Show(CMD_OBS_COORDS, obscoords);
	m_Menu.Enable(CMD_CONVERT, status.files>0);
	m_Menu.Enable(CMD_REMOVE_FILES, selected>0);
	m_Menu.EnableMenu(MENU_CORRECTIONS, status.converted>0);
	m_Menu.EnableMenu(MENU_PHOTOMETRY, status.converted>0);
	m_Menu.EnableMenu(MENU_MATCHING, status.photometred>0);
	m_Menu.EnableMenu(MENU_PLOTTING, status.matched>0);
	m_Menu.Enable(CMD_MASTER_BIAS, status.converted>0);
	m_Menu.Enable(CMD_MASTER_DARK, status.converted>0);
	m_Menu.Enable(CMD_MASTER_FLAT, status.converted>0);
	m_Menu.Enable(CMD_OBJ_COORDS, status.matched>0);
	m_Menu.Enable(CMD_OBS_COORDS, status.matched>0);
	m_Menu.Enable(CMD_PLOT_TRACK_LIST, status.matched>0);
	m_Menu.Enable(CMD_PLOT_AIRMASS, status.converted>0);
	m_Menu.Enable(CMD_FIND_VARIABLES, status.matched>0);
	m_Menu.Enable(CMD_MAKE_CAT_FILE, status.matched>0);

	// Update toolbar controls
	m_TBar.Show(CMD_BIAS_CORR, advcalib);
	m_TBar.Show(CMD_MASTER_BIAS, advcalib);
	m_TBar.Show(CMD_OBJ_COORDS, objcoords);
	m_TBar.Show(CMD_OBS_COORDS, obscoords);
	m_TBar.Enable(CMD_CONVERT, status.files>0);
	m_TBar.Enable(CMD_TIME_CORR, status.converted>0);
	m_TBar.Enable(CMD_BIAS_CORR, status.converted>0);
	m_TBar.Enable(CMD_DARK_CORR, status.converted>0);
	m_TBar.Enable(CMD_FLAT_CORR, status.converted>0);
	m_TBar.Enable(CMD_PHOTOMETRY, status.converted>0);
	m_TBar.Enable(CMD_MATCHING, status.photometred>0);
	m_TBar.Enable(CMD_CHOOSE_STARS, status.matched>0);
	m_TBar.Enable(CMD_CHOOSE_APER, status.matched>0);
	m_TBar.Enable(CMD_OBJ_COORDS, status.matched>0);
	m_TBar.Enable(CMD_OBS_COORDS, status.matched>0);
	m_TBar.Enable(CMD_PLOT_LIGHT_CURVE, status.matched>0);
	m_TBar.Enable(CMD_PLOT_CHART, status.matched>0);
	m_TBar.Enable(CMD_FIND_VARIABLES, status.matched>0);

	// Update status bar
	if (selected==0)
		m_SBar.Set(NULL);
	else if (selected==1) {
		fpath = NULL;
		gtk_tree_selection_selected_foreach(pSel, 
			GtkTreeSelectionForeachFunc(foreach_selected), &fpath);
		m_SBar.Set(fpath);
		g_free(fpath);
	} else {
		sprintf(buf, "%d selected files", selected);
		m_SBar.Set(buf);
	}
}

gint CMainWindow::button_press_event(GtkWidget *widget, GdkEventButton *event, CMainWindow *pMe)
{
	if (event->type==GDK_BUTTON_PRESS && event->button==3) {
		// Show context menu
		return TRUE;
	}
	return FALSE;
}


void CMainWindow::BeginUpdate(void)
{
	assert(m_LockCounter >= 0);
	if (m_LockCounter==0) {
		gtk_tree_view_set_model(GTK_TREE_VIEW(m_TreeView), NULL);
		gtk_icon_view_set_model(GTK_ICON_VIEW(m_IconView), NULL);
	}
	m_LockCounter++;
}

void CMainWindow::EndUpdate(void)
{
	assert(m_LockCounter > 0);
	m_LockCounter--;
	if (m_LockCounter==0) {
		gtk_tree_view_set_model(GTK_TREE_VIEW(m_TreeView), Global.pProject->FileList());
		gtk_icon_view_set_model(GTK_ICON_VIEW(m_IconView), Global.pProject->FileList());
	}
}

GdkPixbuf *CMainWindow::GetIconRef(int icon_id)
{
	int i;

	for (i=0; TreeViewIcons[i].name!=NULL; i++) {
		if (TreeViewIcons[i].id == icon_id)
			return m_Icons[i];
	}
	return NULL;
}

void CMainWindow::ShowTable()
{
	if (m_ShowIcons) {
		m_ShowIcons = false;
		gtk_widget_hide(m_IconScrWnd);
		gtk_widget_show(m_TreeScrWnd);
		gtk_widget_queue_resize(m_Box);
		CConfig::SetBool("Environment", "IconView", false);
	}
}

void CMainWindow::ShowIcons()
{
	if (!m_ShowIcons) {
		m_ShowIcons = true;
		gtk_widget_hide(m_TreeScrWnd);
		gtk_widget_show(m_IconScrWnd);
		gtk_widget_queue_resize(m_Box);
		CConfig::SetBool("Environment", "IconView", true);
	}
}
