/**************************************************************

main_dlg.h (C-Munipack project)
Main dialog
Copyright (C) 2003 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#ifndef CMPACK_MAIN_DLG_H
#define CMPACK_MAIN_DLG_H

#include <gtk/gtk.h>

#include "toolbar.h"
#include "menubar.h"
#include "statusbar.h"
#include "frame_dlg.h"
#include "file_dlg.h"

class CMainWindow
{
public:
	CMainWindow();
	~CMainWindow();

	GtkTreeSelection *GetSelection(void)
	{ return gtk_tree_view_get_selection(GTK_TREE_VIEW(m_TreeView)); }

	GtkWindow *GetHandle(void)
	{ return m_pDlg; }

	GdkPixbuf *GetIconRef(int icon_id);

	void BeginUpdate(void);
	void EndUpdate(void);

	void ShowFrame(GtkTreePath *path);
	void FrameClosed(CFrameDlg *pDlg);

	void ShowFile(const char *path);
	void FileClosed(CFileDlg *pDlg);

private:
	CToolBar	m_TBar;
	CMenuBar	m_Menu;
	CStatusBar	m_SBar;
	GtkWindow	*m_pDlg;
	GtkWidget	*m_TreeView, *m_IconView, *m_Box;
	GtkWidget	*m_TreeScrWnd, *m_IconScrWnd;
	GdkPixbuf	**m_Icons;
	GSList		*m_Previews;
	GSList		*m_Files;
	bool		m_ShowIcons;
	int			m_LockCounter;

	void OnCommand(int cmd_id);
	void OnRowActivated(GtkTreeView *tree_view, GtkTreePath *path);
	void OnItemActivated(GtkIconView *iconview, GtkTreePath *path);
	class CFrameDlg *FindFrame(GtkTreePath *pPath);
	class CFileDlg *FindFile(const char *fpath);
	void UpdateControls();
	void RebuildTable();
	void ShowIcons();
	void ShowTable();

	static gint exit_event(GtkWidget *widget, GdkEvent *event, CMainWindow *pMe);
	static void row_activated(GtkTreeView *treeview, GtkTreePath *path, GtkTreeViewColumn *column, CMainWindow *pMe);
	static void item_activated(GtkIconView *iconview, GtkTreePath *path, CMainWindow *pMe);
	static void MenuCallback(int message, int wparam, void* lparam, void* cb_data);
	static void ToolbarCallback(int message, int wparam, void* lparam, void* cb_data);
	static void selection_changed(GtkTreeSelection *widget, CMainWindow *pMe);
	static gint button_press_event(GtkWidget *widget, GdkEventButton *event, CMainWindow *pMe);

	void ExitApplication(void);
	void AddFiles(void);
	void AddFolder(void);
	void ClearProject(void);
	void RemoveFiles(void);
	void ConvertFiles(void);
	void OpenFile(void);

	void TimeCorrection(void);
	void BiasCorrection(void);
	void DarkCorrection(void);
	void FlatCorrection(void);

	void Photometry(void);

	void Matching(void);

	void ChooseStars(void);
	void ChooseAperture(void);
	void EnterObjectCoords(void);
	void EnterObserverCoords(void);
	void PlotLightCurve(void);
	void PlotChart(void);

	void FindVariables(void);
	void MakeCatalogFile(void);
	void MasterBias(void);
	void MasterDark(void);
	void MasterFlat(void);
	void PlotTrackList(void);
	void PlotAirMass(void);
	void ExpressReduction(void);
	void NewFiles(void);
	void JDConverter(void);
	void HelCorrection(void);
	void AirMass(void);
	void EditPreferences(void);

	void ShowMessages(void);
	void ShowProject(void);

	void AboutApplication(void);
};

#endif
