/**************************************************************

object_dlg.cpp (C-Munipack project)
The 'Enter object coordinates' dialog
Copyright (C) 2009 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#include <stdlib.h>
#include <string.h>
#include <glib.h>
#include <cmunipack.h>

#include "utils.h"
#include "configuration.h"
#include "object_dlg.h"
#include "main.h"
#include "varcat.h"

enum tColumnId
{
	SEARCH_COL_ID,
	SEARCH_COL_NAME,
	SEARCH_COL_RA,
	SEARCH_COL_DEC,
	SEARCH_COL_SRC,
	SEARCH_COL_REM,
	SEARCH_NCOLS
};

const static struct tTreeViewColumn {
	const char *caption;		// Column name
	int column;					// Model column index
	double align;				// Column alignment
} TreeViewColumns[] = {
	{ "Designation",		SEARCH_COL_NAME },
	{ "Right ascension",	SEARCH_COL_RA, 1.0 },
	{ "Declination",		SEARCH_COL_DEC, 1.0 },
	{ "Source",				SEARCH_COL_SRC },
	{ "Remarks",			SEARCH_COL_REM },
	{ NULL }
};

struct tColData
{
	GtkTreeViewColumn *col;
	const tTreeViewColumn *data;
};

//-------------------------   JD CONVERTER DIALOG   --------------------------------

CObjectDlg::CObjectDlg(GtkWindow *pParent):m_Columns(NULL)
{
	int i;
	GtkWidget *label, *vbox, *scrwnd, *mainbox;
	GSList *group;

	// Search result
	m_SearchRes = gtk_list_store_new(SEARCH_NCOLS, G_TYPE_INT, G_TYPE_STRING, G_TYPE_STRING, 
		G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);

	// Dialog with buttons
	m_pDlg = gtk_dialog_new_with_buttons("Enter object's coordinates", pParent, 
		(GtkDialogFlags)(GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_NO_SEPARATOR),
		GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT, GTK_STOCK_OK, GTK_RESPONSE_ACCEPT, NULL);

	// Initial state
	m_SelectMode = (tSelectMode)CConfig::GetInt("ObjectDlg", "Select", REFERENCE_FRAME);
	m_SortColumn = CConfig::GetInt("ObjectDlg", "Sort", 0);

	// Dialog icon
	gchar *icon = get_icon_file("objcoords");
	gtk_window_set_icon(GTK_WINDOW(m_pDlg), gdk_pixbuf_new_from_file(icon, NULL));
	g_free(icon);

	// Dialog layout
	mainbox = GTK_DIALOG(m_pDlg)->vbox;

	// Method
	m_Select = gtk_frame_new("Get object coordinates");
	gtk_box_pack_start(GTK_BOX(mainbox), m_Select, FALSE, TRUE, 0);
	vbox = gtk_vbox_new(TRUE, 4);
	gtk_container_set_border_width(GTK_CONTAINER(vbox), 4);
	gtk_container_add(GTK_CONTAINER(m_Select), vbox);
	m_RefBtn = gtk_radio_button_new_with_label(NULL, "from the reference frame");
	gtk_box_pack_start(GTK_BOX(vbox), m_RefBtn, TRUE, TRUE, 0);
	group = gtk_radio_button_get_group(GTK_RADIO_BUTTON(m_RefBtn)); 
	m_ManBtn = gtk_radio_button_new_with_label(group, "enter coordinates manually");
	gtk_box_pack_start(GTK_BOX(vbox), m_ManBtn, TRUE, TRUE, 0);
	group = gtk_radio_button_get_group(GTK_RADIO_BUTTON(m_RefBtn)); 
	m_CatBtn = gtk_radio_button_new_with_label(group, "from catalogues of variable stars");
	gtk_box_pack_start(GTK_BOX(vbox), m_CatBtn, TRUE, TRUE, 0);

	// Coordinates from the referece file
	m_AutoBox = gtk_table_new(3, 3, FALSE);
	gtk_box_pack_start(GTK_BOX(mainbox), m_AutoBox, FALSE, TRUE, 0);
	gtk_container_set_border_width(GTK_CONTAINER(m_AutoBox), 8);
	gtk_table_set_row_spacings(GTK_TABLE(m_AutoBox), 8);
	gtk_table_set_col_spacings(GTK_TABLE(m_AutoBox), 8);
	label = gtk_label_new("Designation");
	gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(m_AutoBox), label, 0, 1, 0, 1);
	m_AutoName = gtk_entry_new_with_max_length(256);
	gtk_widget_set_sensitive(m_AutoName, false);
	gtk_table_attach_defaults(GTK_TABLE(m_AutoBox), m_AutoName, 1, 2, 0, 1);
	label = gtk_label_new("Right ascension");
	gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(m_AutoBox), label, 0, 1, 1, 2);
	m_AutoRA = gtk_entry_new_with_max_length(32);
	gtk_widget_set_sensitive(m_AutoRA, false);
	gtk_table_attach_defaults(GTK_TABLE(m_AutoBox), m_AutoRA, 1, 2, 1, 2);
	label = gtk_label_new("[hh mm ss]");
	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(m_AutoBox), label, 2, 3, 1, 2);
	label = gtk_label_new("- declination");
	gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(m_AutoBox), label, 0, 1, 2, 3);
	m_AutoDec = gtk_entry_new_with_max_length(32);
	gtk_widget_set_sensitive(m_AutoDec, false);
	gtk_table_attach_defaults(GTK_TABLE(m_AutoBox), m_AutoDec, 1, 2, 2, 3);
	label = gtk_label_new("[\xC2\xB1""ddd mm ss]");
	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(m_AutoBox), label, 2, 3, 2, 3);

	// Object coordinates display
	m_EntryBox = gtk_table_new(3, 3, FALSE);
	gtk_box_pack_start(GTK_BOX(mainbox), m_EntryBox, FALSE, TRUE, 0);
	gtk_container_set_border_width(GTK_CONTAINER(m_EntryBox), 8);
	gtk_table_set_row_spacings(GTK_TABLE(m_EntryBox), 8);
	gtk_table_set_col_spacings(GTK_TABLE(m_EntryBox), 8);
	label = gtk_label_new("Designation");
	gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(m_EntryBox), label, 0, 1, 0, 1);
	m_EntryName = gtk_entry_new_with_max_length(256);
	gtk_table_attach_defaults(GTK_TABLE(m_EntryBox), m_EntryName, 1, 2, 0, 1);
	label = gtk_label_new("Right ascension");
	gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(m_EntryBox), label, 0, 1, 1, 2);
	m_EntryRA = gtk_entry_new_with_max_length(32);
	gtk_table_attach_defaults(GTK_TABLE(m_EntryBox), m_EntryRA, 1, 2, 1, 2);
	label = gtk_label_new("[hh mm ss]");
	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(m_EntryBox), label, 2, 3, 1, 2);
	label = gtk_label_new("- declination");
	gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(m_EntryBox), label, 0, 1, 2, 3);
	m_EntryDec = gtk_entry_new_with_max_length(32);
	gtk_table_attach_defaults(GTK_TABLE(m_EntryBox), m_EntryDec, 1, 2, 2, 3);
	label = gtk_label_new("[\xC2\xB1""ddd mm ss]");
	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(m_EntryBox), label, 2, 3, 2, 3);

	// Search box
	m_SearchBox = gtk_table_new(2, 3, FALSE);
	gtk_box_pack_start(GTK_BOX(mainbox), m_SearchBox, TRUE, TRUE, 0);
	gtk_container_set_border_width(GTK_CONTAINER(m_SearchBox), 8);
	gtk_table_set_row_spacings(GTK_TABLE(m_SearchBox), 8);
	gtk_table_set_col_spacings(GTK_TABLE(m_SearchBox), 8);
	label = gtk_label_new("Search for");
	gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
	gtk_table_attach(GTK_TABLE(m_SearchBox), label, 0, 1, 0, 1, GTK_FILL, GTK_FILL, 0, 0);
	m_FindText = gtk_entry_new_with_max_length(32);
	gtk_table_attach(GTK_TABLE(m_SearchBox), m_FindText, 1, 2, 0, 1, 
		(GtkAttachOptions)(GTK_FILL | GTK_EXPAND), GTK_FILL, 0, 0);
	m_FindBtn = gtk_button_new_with_label("Search");
	gtk_table_attach(GTK_TABLE(m_SearchBox), m_FindBtn, 2, 3, 0, 1, GTK_FILL, GTK_FILL, 0, 0);
	m_SearchView = gtk_tree_view_new();
	for (i=0; TreeViewColumns[i].caption!=NULL; i++) {
		GtkTreeViewColumn *col = gtk_tree_view_column_new();
		// Set column name and alignment
		gtk_tree_view_column_set_title(col, TreeViewColumns[i].caption);
		gtk_tree_view_append_column(GTK_TREE_VIEW(m_SearchView), col);
		// Add text renderer
		GtkCellRenderer *renderer = gtk_cell_renderer_text_new();
		gtk_tree_view_column_pack_start(col, renderer, TRUE);
		g_object_set(renderer, "xalign", TreeViewColumns[i].align, NULL);
		gtk_tree_view_column_add_attribute(col, renderer, "text", TreeViewColumns[i].column);
		g_signal_connect(G_OBJECT(col), "clicked", G_CALLBACK(column_clicked), this);
		tColData *data = (tColData*)g_malloc(sizeof(tColData));
		data->col = col;
		data->data = &TreeViewColumns[i];
		m_Columns = g_slist_append(m_Columns, data);
	}
	gtk_tree_view_set_model(GTK_TREE_VIEW(m_SearchView), GTK_TREE_MODEL(m_SearchRes));
	gtk_tree_view_set_headers_clickable(GTK_TREE_VIEW(m_SearchView), true);
	gtk_tree_selection_set_mode(gtk_tree_view_get_selection(GTK_TREE_VIEW(m_SearchView)), GTK_SELECTION_SINGLE);
	scrwnd = gtk_scrolled_window_new (NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrwnd),
		GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrwnd), 
		GTK_SHADOW_ETCHED_IN);
	gtk_container_add(GTK_CONTAINER(scrwnd), m_SearchView);
	gtk_table_attach_defaults(GTK_TABLE(m_SearchBox), scrwnd, 0, 3, 1, 2);

	// Signals:
	g_signal_connect(G_OBJECT(m_pDlg), "response", G_CALLBACK(response_dialog), this);
	g_signal_connect(G_OBJECT(m_RefBtn), "toggled", G_CALLBACK(button_clicked), this);
	g_signal_connect(G_OBJECT(m_ManBtn), "toggled", G_CALLBACK(button_clicked), this);
	g_signal_connect(G_OBJECT(m_CatBtn), "toggled", G_CALLBACK(button_clicked), this);
	g_signal_connect(G_OBJECT(m_FindBtn), "clicked", G_CALLBACK(button_clicked), this);

	gtk_widget_set_size_request(mainbox, 400, 400);
	gtk_widget_show_all(mainbox);
	SetSelectMode(m_SelectMode);
	SetSortColumn(m_SortColumn);
}

CObjectDlg::~CObjectDlg()
{
	for (GSList *ptr=m_Columns; ptr!=NULL; ptr=ptr->next)
		g_free(ptr->data);
	g_slist_free(m_Columns);
	gtk_widget_destroy(m_pDlg);
}

bool CObjectDlg::Execute()
{
	bool ref_ok, def_ok;
	double ra, dec;
	char buf[64];

	// User coordinates
	const CObjectCoords *pObj = Global.pProject->ObjectCoords();
	if (pObj->Name()) 
		gtk_entry_set_text(GTK_ENTRY(m_EntryName), pObj->Name());
	else
		gtk_entry_set_text(GTK_ENTRY(m_EntryName), "");
	if (cmpack_strtora(pObj->RA(), &ra)==0) {
		cmpack_ratostr(ra, buf, 64);
		gtk_entry_set_text(GTK_ENTRY(m_EntryRA), buf);
	} else
		gtk_entry_set_text(GTK_ENTRY(m_EntryRA), "");
	if (cmpack_strtodec(pObj->Dec(), &dec)==0) {
		cmpack_dectostr(dec, buf, 64);
		gtk_entry_set_text(GTK_ENTRY(m_EntryDec), buf);
	} else
		gtk_entry_set_text(GTK_ENTRY(m_EntryDec), "");

	// Reference coordinates
	pObj = Global.pProject->ReferenceObjectCoords();
	if (pObj->Name()) 
		gtk_entry_set_text(GTK_ENTRY(m_AutoName), pObj->Name());
	else
		gtk_entry_set_text(GTK_ENTRY(m_AutoName), "");
	if (cmpack_strtora(pObj->RA(), &ra)==0) {
		cmpack_ratostr(ra, buf, 64);
		gtk_entry_set_text(GTK_ENTRY(m_AutoRA), buf);
	} else
		gtk_entry_set_text(GTK_ENTRY(m_AutoRA), "");
	if (cmpack_strtodec(pObj->Dec(), &dec)==0) {
		cmpack_dectostr(dec, buf, 64);
		gtk_entry_set_text(GTK_ENTRY(m_AutoDec), buf);
	} else
		gtk_entry_set_text(GTK_ENTRY(m_AutoDec), "");
	ref_ok = cmpack_strtora(pObj->RA(), &ra)==0 &&
		cmpack_strtodec(pObj->Dec(), &dec)==0;
	gtk_widget_set_sensitive(m_RefBtn, ref_ok);

	// Coordinates from catalogue file
	def_ok = VarCat_Test();
	gtk_widget_set_sensitive(m_CatBtn, def_ok);

	if (ref_ok && m_SelectMode==REFERENCE_FRAME) {
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(m_CatBtn), false);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(m_ManBtn), false);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(m_RefBtn), true);
		SetSelectMode(REFERENCE_FRAME);
	} else
	if (def_ok && m_SelectMode==CATALOGUE_FILE) {
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(m_RefBtn), false);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(m_ManBtn), false);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(m_CatBtn), true);
		SetSelectMode(CATALOGUE_FILE);
	} else {
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(m_RefBtn), false);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(m_CatBtn), false);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(m_ManBtn), true);
		SetSelectMode(MANUAL_ENTRY);
	}
	
	return gtk_dialog_run(GTK_DIALOG(m_pDlg))==GTK_RESPONSE_ACCEPT;
}

void CObjectDlg::response_dialog(GtkDialog *pDlg, gint response_id, CObjectDlg *pMe)
{
	if (!pMe->OnResponseDialog(response_id))
		g_signal_stop_emission_by_name(pDlg, "response");
}

bool CObjectDlg::OnResponseDialog(gint response_id)
{
	gchar *name, *ra, *dec;
	char buf[64];
	double x, y;
	GtkTreeIter iter;

	if (response_id==GTK_RESPONSE_ACCEPT) {
		name = ra = dec = NULL;
		if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(m_RefBtn))) {
			name = g_strdup(gtk_entry_get_text(GTK_ENTRY(m_AutoName)));
			ra = g_strdup(gtk_entry_get_text(GTK_ENTRY(m_AutoRA)));
			dec = g_strdup(gtk_entry_get_text(GTK_ENTRY(m_AutoDec)));
		} else if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(m_ManBtn))) {
			name = g_strdup(gtk_entry_get_text(GTK_ENTRY(m_EntryName)));
			ra = g_strdup(gtk_entry_get_text(GTK_ENTRY(m_EntryRA)));
			dec = g_strdup(gtk_entry_get_text(GTK_ENTRY(m_EntryDec)));
		} else if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(m_CatBtn))) {
			GtkTreeSelection *selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(m_SearchView));
			if (gtk_tree_selection_get_selected(selection, NULL, &iter)) {
				gtk_tree_model_get(GTK_TREE_MODEL(m_SearchRes), &iter, SEARCH_COL_NAME,
					&name, SEARCH_COL_RA, &ra, SEARCH_COL_DEC, &dec, -1);
			} else {
				ShowError(GTK_WINDOW(m_pDlg), "Please, select a row in the table of objects.");
				return false;
			}
		}
		if (cmpack_strtora(ra, &x)!=0) {
			ShowError(GTK_WINDOW(m_pDlg), "Invalid value of right ascension.");
			return false;
		}
		if (cmpack_strtodec(dec, &y)!=0) {
			ShowError(GTK_WINDOW(m_pDlg), "Invalid value of declination.");
			return false;
		}
		Global.pProject->ObjectCoords()->SetName(name);
		cmpack_ratostr(x, buf, 64);
		Global.pProject->ObjectCoords()->SetRA(buf);
		cmpack_dectostr(y, buf, 64);
		Global.pProject->ObjectCoords()->SetDec(buf);
		g_free(name);
		g_free(ra);
		g_free(dec);
		Global.pProject->Save();
	}
	return true;
}

void CObjectDlg::column_clicked(GtkTreeViewColumn *treeviewcolumn, CObjectDlg *pDlg)
{
	pDlg->OnColumnClicked(treeviewcolumn);
}

void CObjectDlg::OnColumnClicked(GtkTreeViewColumn *pCol)
{
	GSList *ptr;

	for (ptr=m_Columns; ptr!=NULL; ptr=ptr->next) {
		tColData *col = (tColData*)ptr->data;
		if (col->col == pCol) {
			SetSortColumn(col->data->column);
			CConfig::SetInt("ObjectDlg", "Sort", m_SortColumn);
			break;
		}
	}
}

void CObjectDlg::button_clicked(GtkWidget *button, CObjectDlg *pDlg)
{
	pDlg->OnButtonClicked(button);
}

void CObjectDlg::OnButtonClicked(GtkWidget *pBtn)
{
	if (pBtn==m_RefBtn)
		SetSelectMode(REFERENCE_FRAME);
	else if (pBtn==m_ManBtn)
		SetSelectMode(MANUAL_ENTRY);
	else if (pBtn==m_CatBtn)
		SetSelectMode(CATALOGUE_FILE);
	else if (pBtn==m_FindBtn)
		FindObjects();
}

void CObjectDlg::SetSelectMode(tSelectMode mode)
{
	m_SelectMode = mode;
	switch (mode)
	{
	case REFERENCE_FRAME:
		gtk_widget_hide(m_SearchBox);
		gtk_widget_hide(m_EntryBox);
		gtk_widget_show(m_AutoBox);
		break;
	case MANUAL_ENTRY:
		gtk_widget_hide(m_SearchBox);
		gtk_widget_hide(m_AutoBox);
		gtk_widget_show(m_EntryBox);
		break;
	case CATALOGUE_FILE:
		gtk_widget_hide(m_EntryBox);
		gtk_widget_hide(m_AutoBox);
		gtk_widget_show(m_SearchBox);
		break;
	}
}

void CObjectDlg::SetSortColumn(int column)
{
	for (GSList *ptr=m_Columns; ptr!=NULL; ptr=ptr->next) {
		tColData *col = (tColData*)ptr->data;
		if (col->data->column == column) {
			m_SortColumn = column;
			gtk_tree_view_column_set_sort_indicator(col->col, true);
			gtk_tree_view_column_set_sort_order(col->col, GTK_SORT_ASCENDING);
			gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(m_SearchRes), column, GTK_SORT_ASCENDING);
		} else {
			m_SortColumn = -1;
			gtk_tree_view_column_set_sort_indicator(col->col, false);
		}
	}
}

void CObjectDlg::FindObjects(void)
{
	char *text = g_locale_from_utf8(gtk_entry_get_text(GTK_ENTRY(m_FindText)), -1, NULL, NULL, NULL);
	if (strlen(text)>=2) {
		gtk_tree_view_set_model(GTK_TREE_VIEW(m_SearchView), NULL);
		if (strlen(text)>0) {
			m_Counter = 0;
			gtk_list_store_clear(m_SearchRes);
			VarCat_Search(text, (tVarCatProc*)AddToList, this);
		}
		gtk_tree_view_set_model(GTK_TREE_VIEW(m_SearchView), GTK_TREE_MODEL(m_SearchRes));
	} else {
		ShowError(GTK_WINDOW(m_pDlg), "Please, enter at least 2 characters as a search pattern.");
	}
	g_free(text);
}

void CObjectDlg::AddToList(const char *objname, const char *ra, const char *dec,
		const char *catalogue, const char *comment, CObjectDlg *data)
{
	GtkTreeIter iter;
	gtk_list_store_append(data->m_SearchRes, &iter);
	gtk_list_store_set(data->m_SearchRes, &iter, SEARCH_COL_ID, data->m_Counter++,
		SEARCH_COL_NAME, objname, SEARCH_COL_RA, ra, SEARCH_COL_DEC, dec, 
		SEARCH_COL_SRC, catalogue, SEARCH_COL_REM, comment, -1);
}
