/**************************************************************

observer_dlg.cpp (C-Munipack project)
The 'Enter observer coordinates' dialog
Copyright (C) 2009 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#include <stdlib.h>
#include <string.h>
#include <glib.h>
#include <cmunipack.h>

#include "utils.h"
#include "configuration.h"
#include "observer_dlg.h"
#include "main.h"
#include "varcat.h"

//-------------------------   DIALOG   --------------------------------

CObserverDlg::CObserverDlg(GtkWindow *pParent)
{
	GtkWidget *label, *vbox, *mainbox;
	GSList *group;

	// Dialog with buttons
	m_pDlg = gtk_dialog_new_with_buttons("Enter observer's coordinates", pParent, 
		(GtkDialogFlags)(GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_NO_SEPARATOR),
		GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT, GTK_STOCK_OK, GTK_RESPONSE_ACCEPT, NULL);

	// Initial state
	m_SelectMode = (tSelectMode)CConfig::GetInt("ObserverDlg", "Select", DEFAULT_LOCATION);

	// Dialog icon
	gchar *icon = get_icon_file("obscoords");
	gtk_window_set_icon(GTK_WINDOW(m_pDlg), gdk_pixbuf_new_from_file(icon, NULL));
	g_free(icon);

	// Dialog layout
	mainbox = GTK_DIALOG(m_pDlg)->vbox;

	// Method
	m_Select = gtk_frame_new("Get observer coordinates");
	gtk_box_pack_start(GTK_BOX(mainbox), m_Select, FALSE, TRUE, 0);
	vbox = gtk_vbox_new(TRUE, 4);
	gtk_container_set_border_width(GTK_CONTAINER(vbox), 4);
	gtk_container_add(GTK_CONTAINER(m_Select), vbox);
	m_DefBtn = gtk_radio_button_new_with_label(NULL, "use default location");
	gtk_box_pack_start(GTK_BOX(vbox), m_DefBtn, TRUE, TRUE, 0);
	group = gtk_radio_button_get_group(GTK_RADIO_BUTTON(m_DefBtn)); 
	m_RefBtn = gtk_radio_button_new_with_label(group, "from the reference frame");
	gtk_box_pack_start(GTK_BOX(vbox), m_RefBtn, TRUE, TRUE, 0);
	group = gtk_radio_button_get_group(GTK_RADIO_BUTTON(m_DefBtn)); 
	m_ManBtn = gtk_radio_button_new_with_label(group, "enter coordinates manually");
	gtk_box_pack_start(GTK_BOX(vbox), m_ManBtn, TRUE, TRUE, 0);

	// Default coordinates
	m_DefaultBox = gtk_table_new(3, 3, FALSE);
	gtk_box_pack_start(GTK_BOX(mainbox), m_DefaultBox, FALSE, TRUE, 0);
	gtk_container_set_border_width(GTK_CONTAINER(m_DefaultBox), 8);
	gtk_table_set_row_spacings(GTK_TABLE(m_DefaultBox), 8);
	gtk_table_set_col_spacings(GTK_TABLE(m_DefaultBox), 8);
	label = gtk_label_new("Location");
	gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(m_DefaultBox), label, 0, 1, 0, 1);
	m_DefaultName = gtk_entry_new_with_max_length(256);
	gtk_widget_set_sensitive(m_DefaultName, false);
	gtk_table_attach_defaults(GTK_TABLE(m_DefaultBox), m_DefaultName, 1, 2, 0, 1);
	label = gtk_label_new("Longitude");
	gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(m_DefaultBox), label, 0, 1, 1, 2);
	m_DefaultLon = gtk_entry_new_with_max_length(32);
	gtk_widget_set_sensitive(m_DefaultLon, false);
	gtk_table_attach_defaults(GTK_TABLE(m_DefaultBox), m_DefaultLon, 1, 2, 1, 2);
	label = gtk_label_new("[\xC2\xB1""ddd mm ss]");
	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(m_DefaultBox), label, 2, 3, 1, 2);
	label = gtk_label_new("Latitude");
	gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(m_DefaultBox), label, 0, 1, 2, 3);
	m_DefaultLat = gtk_entry_new_with_max_length(32);
	gtk_widget_set_sensitive(m_DefaultLat, false);
	gtk_table_attach_defaults(GTK_TABLE(m_DefaultBox), m_DefaultLat, 1, 2, 2, 3);
	label = gtk_label_new("[\xC2\xB1""dd mm ss]");
	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(m_DefaultBox), label, 2, 3, 2, 3);

	// Default coordinates
	m_RefBox = gtk_table_new(3, 3, FALSE);
	gtk_box_pack_start(GTK_BOX(mainbox), m_RefBox, FALSE, TRUE, 0);
	gtk_container_set_border_width(GTK_CONTAINER(m_RefBox), 8);
	gtk_table_set_row_spacings(GTK_TABLE(m_RefBox), 8);
	gtk_table_set_col_spacings(GTK_TABLE(m_RefBox), 8);
	label = gtk_label_new("Location");
	gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(m_RefBox), label, 0, 1, 0, 1);
	m_RefName = gtk_entry_new_with_max_length(256);
	gtk_widget_set_sensitive(m_RefName, false);
	gtk_table_attach_defaults(GTK_TABLE(m_RefBox), m_RefName, 1, 2, 0, 1);
	label = gtk_label_new("Longitude");
	gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(m_RefBox), label, 0, 1, 1, 2);
	m_RefLon = gtk_entry_new_with_max_length(32);
	gtk_widget_set_sensitive(m_RefLon, false);
	gtk_table_attach_defaults(GTK_TABLE(m_RefBox), m_RefLon, 1, 2, 1, 2);
	label = gtk_label_new("[\xC2\xB1""ddd mm ss]");
	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(m_RefBox), label, 2, 3, 1, 2);
	label = gtk_label_new("Latitude");
	gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(m_RefBox), label, 0, 1, 2, 3);
	m_RefLat = gtk_entry_new_with_max_length(32);
	gtk_widget_set_sensitive(m_RefLat, false);
	gtk_table_attach_defaults(GTK_TABLE(m_RefBox), m_RefLat, 1, 2, 2, 3);
	label = gtk_label_new("[\xC2\xB1""dd mm ss]");
	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(m_RefBox), label, 2, 3, 2, 3);

	// Manual entry
	m_EntryBox = gtk_table_new(3, 3, FALSE);
	gtk_box_pack_start(GTK_BOX(mainbox), m_EntryBox, FALSE, TRUE, 0);
	gtk_container_set_border_width(GTK_CONTAINER(m_EntryBox), 8);
	gtk_table_set_row_spacings(GTK_TABLE(m_EntryBox), 8);
	gtk_table_set_col_spacings(GTK_TABLE(m_EntryBox), 8);
	label = gtk_label_new("Location");
	gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(m_EntryBox), label, 0, 1, 0, 1);
	m_EntryName = gtk_entry_new_with_max_length(256);
	gtk_table_attach_defaults(GTK_TABLE(m_EntryBox), m_EntryName, 1, 2, 0, 1);
	label = gtk_label_new("Longitude");
	gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(m_EntryBox), label, 0, 1, 1, 2);
	m_EntryLon = gtk_entry_new_with_max_length(32);
	gtk_table_attach_defaults(GTK_TABLE(m_EntryBox), m_EntryLon, 1, 2, 1, 2);
	label = gtk_label_new("[\xC2\xB1""ddd mm ss]");
	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(m_EntryBox), label, 2, 3, 1, 2);
	label = gtk_label_new("Latitude");
	gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(m_EntryBox), label, 0, 1, 2, 3);
	m_EntryLat = gtk_entry_new_with_max_length(32);
	gtk_table_attach_defaults(GTK_TABLE(m_EntryBox), m_EntryLat, 1, 2, 2, 3);
	label = gtk_label_new("[\xC2\xB1""dd mm ss]");
	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(m_EntryBox), label, 2, 3, 2, 3);

	// Signals:
	g_signal_connect(G_OBJECT(m_pDlg), "response", G_CALLBACK(response_dialog), this);
	g_signal_connect(G_OBJECT(m_DefBtn), "toggled", G_CALLBACK(button_clicked), this);
	g_signal_connect(G_OBJECT(m_RefBtn), "toggled", G_CALLBACK(button_clicked), this);
	g_signal_connect(G_OBJECT(m_ManBtn), "toggled", G_CALLBACK(button_clicked), this);

	gtk_widget_set_size_request(mainbox, 400, 400);
	gtk_widget_show_all(mainbox);
	SetSelectMode(m_SelectMode);
}

CObserverDlg::~CObserverDlg()
{
	gtk_widget_destroy(m_pDlg);
}

bool CObserverDlg::Execute()
{
	bool ref_ok, def_ok;
	double lon, lat;
	char buf[64];

	CObserverCoords usr(*Global.pProject->UserLocation());
	if (usr.Name()) 
		gtk_entry_set_text(GTK_ENTRY(m_EntryName), usr.Name());
	else
		gtk_entry_set_text(GTK_ENTRY(m_EntryName), "");
	if (cmpack_strtolon(usr.Lon(), &lon)==0) {
		cmpack_lontostr(lon, buf, 64);
		gtk_entry_set_text(GTK_ENTRY(m_EntryLon), buf);
	} else
		gtk_entry_set_text(GTK_ENTRY(m_EntryLon), "");
	if (cmpack_strtolat(usr.Lat(), &lat)==0) {
		cmpack_lattostr(lat, buf, 64);
		gtk_entry_set_text(GTK_ENTRY(m_EntryLat), buf);
	} else
		gtk_entry_set_text(GTK_ENTRY(m_EntryLat), "");

	CObserverCoords ref(*Global.pProject->ReferenceLocation());
	if (ref.Name()) 
		gtk_entry_set_text(GTK_ENTRY(m_RefName), ref.Name());
	else
		gtk_entry_set_text(GTK_ENTRY(m_RefName), "");
	if (cmpack_strtolon(ref.Lon(), &lon)==0) {
		cmpack_lontostr(lon, buf, 64);
		gtk_entry_set_text(GTK_ENTRY(m_RefLon), buf);
	} else
		gtk_entry_set_text(GTK_ENTRY(m_RefLon), "");
	if (cmpack_strtolat(ref.Lat(), &lat)==0) {
		cmpack_lattostr(lat, buf, 64);
		gtk_entry_set_text(GTK_ENTRY(m_RefLat), buf);
	} else
		gtk_entry_set_text(GTK_ENTRY(m_RefLat), "");
	ref_ok = cmpack_strtolon(ref.Lon(), &lon)==0 &&
		cmpack_strtolat(ref.Lat(), &lat)==0;
	gtk_widget_set_sensitive(m_RefBtn, ref_ok);

	CObserverCoords def(Global.pProject->DefaultLocation());
	if (def.Name()) 
		gtk_entry_set_text(GTK_ENTRY(m_DefaultName), def.Name());
	else
		gtk_entry_set_text(GTK_ENTRY(m_DefaultName), "");
	if (cmpack_strtolon(def.Lon(), &lon)==0) {
		cmpack_lontostr(lon, buf, 64);
		gtk_entry_set_text(GTK_ENTRY(m_DefaultLon), buf);
	} else
		gtk_entry_set_text(GTK_ENTRY(m_DefaultLon), "");
	if (cmpack_strtolat(def.Lat(), &lat)==0) {
		cmpack_lattostr(lat, buf, 64);
		gtk_entry_set_text(GTK_ENTRY(m_DefaultLat), buf);
	} else
		gtk_entry_set_text(GTK_ENTRY(m_DefaultLat), "");
	def_ok = cmpack_strtolon(def.Lon(), &lon)==0 &&
		cmpack_strtolat(def.Lat(), &lat)==0;
	gtk_widget_set_sensitive(m_DefBtn, def_ok);

	if (def_ok && Global.pProject->UseDefaultLocation()) {
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(m_ManBtn), false);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(m_RefBtn), false);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(m_DefBtn), true);
		SetSelectMode(DEFAULT_LOCATION);
	} else if (ref_ok && m_SelectMode==REFERENCE_FRAME) {
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(m_ManBtn), false);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(m_DefBtn), false);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(m_RefBtn), true);
		SetSelectMode(REFERENCE_FRAME);
	} else {
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(m_RefBtn), false);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(m_DefBtn), false);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(m_ManBtn), true);
		SetSelectMode(MANUAL_ENTRY);
	}

	return gtk_dialog_run(GTK_DIALOG(m_pDlg))==GTK_RESPONSE_ACCEPT;
}

void CObserverDlg::response_dialog(GtkDialog *pDlg, gint response_id, CObserverDlg *pMe)
{
	if (!pMe->OnResponseDialog(response_id))
		g_signal_stop_emission_by_name(pDlg, "response");
}

bool CObserverDlg::OnResponseDialog(gint response_id)
{
	const gchar *name, *lon, *lat;
	char buf[64];
	double x, y;

	if (response_id==GTK_RESPONSE_ACCEPT) {
		name = lon = lat = NULL;
		if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(m_DefBtn))) {
			// Use default location
			Global.pProject->SetUseDefaultLocation(true);
		} else {
			if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(m_RefBtn))) {
				// Reference location
				name = g_strdup(gtk_entry_get_text(GTK_ENTRY(m_RefName)));
				lon = g_strdup(gtk_entry_get_text(GTK_ENTRY(m_RefLon)));
				lat = g_strdup(gtk_entry_get_text(GTK_ENTRY(m_RefLat)));
			} else {
				// User location
				name = gtk_entry_get_text(GTK_ENTRY(m_EntryName));
				lon = gtk_entry_get_text(GTK_ENTRY(m_EntryLon));
				lat = gtk_entry_get_text(GTK_ENTRY(m_EntryLat));
			}
			if (cmpack_strtolon(lon, &x)!=0) {
				ShowError(GTK_WINDOW(m_pDlg), "Invalid value of longitude");
				return false;
			}
			if (cmpack_strtolat(lat, &y)!=0) {
				ShowError(GTK_WINDOW(m_pDlg), "Invalid value of latitude");
				return false;
			}
			Global.pProject->SetUseDefaultLocation(false);
			Global.pProject->UserLocation()->SetName(name);
			cmpack_lontostr(x, buf, 64);
			Global.pProject->UserLocation()->SetLon(buf);
			cmpack_lattostr(y, buf, 64);
			Global.pProject->UserLocation()->SetLat(buf);
		}
		Global.pProject->Save();
	}
	return true;
}

void CObserverDlg::button_clicked(GtkWidget *button, CObserverDlg *pDlg)
{
	pDlg->OnButtonClicked(button);
}

void CObserverDlg::OnButtonClicked(GtkWidget *pBtn)
{
	if (pBtn==m_DefBtn)
		SetSelectMode(DEFAULT_LOCATION);
	else if (pBtn==m_ManBtn)
		SetSelectMode(MANUAL_ENTRY);
}

void CObserverDlg::SetSelectMode(tSelectMode mode)
{
	m_SelectMode = mode;
	switch (mode)
	{
	case DEFAULT_LOCATION:
		gtk_widget_hide(m_RefBox);
		gtk_widget_hide(m_EntryBox);
		gtk_widget_show(m_DefaultBox);
		break;
	case REFERENCE_FRAME:
		gtk_widget_hide(m_EntryBox);
		gtk_widget_hide(m_DefaultBox);
		gtk_widget_show(m_RefBox);
		break;
	case MANUAL_ENTRY:
		gtk_widget_hide(m_RefBox);
		gtk_widget_hide(m_DefaultBox);
		gtk_widget_show(m_EntryBox);
		break;
	}
}
