/**************************************************************

addfiles_dlg.cpp (C-Munipack project)
The 'Add files' dialog
Copyright (C) 2008 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#include <stdlib.h>
#include <glib.h>

#include "open_dlg.h"
#include "configuration.h"
#include "main.h"
#include "utils.h"

COpenDlg::COpenDlg(GtkWindow *pParent)
{
	// Dialog with buttons
	m_pDlg = gtk_file_chooser_dialog_new("Open file", pParent, GTK_FILE_CHOOSER_ACTION_OPEN,
		GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT, GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT, NULL);

	// Dialog icon
	gchar *icon = get_icon_file("fileopen");
	gtk_window_set_icon(GTK_WINDOW(m_pDlg), gdk_pixbuf_new_from_file(icon, NULL));
	g_free(icon);

	// Preview widget
	gtk_file_chooser_set_preview_widget(GTK_FILE_CHOOSER(m_pDlg), 
		GTK_WIDGET(g_object_ref(m_Preview.Handle())));
	m_Preview.SetTypeMask(FILE_TYPE_ALL);
	m_Preview.SetParent(GTK_WINDOW(m_pDlg));

	// Signals:
	g_signal_connect(G_OBJECT(m_pDlg), "response", G_CALLBACK(response_dialog), this);
	g_signal_connect(G_OBJECT(m_pDlg), "update-preview", G_CALLBACK(update_preview), this);
	g_signal_connect(G_OBJECT(m_pDlg), "selection-changed", G_CALLBACK(selection_changed), this);
}

COpenDlg::~COpenDlg()
{
	gtk_widget_destroy(m_pDlg);
}

void COpenDlg::Execute(void)
{
	char *path = CConfig::GetStr("OpenFile", "File", NULL);
	if (path) {
		gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(m_pDlg), path);
		m_Preview.Update(path);
	} else {
		m_Preview.Clear();
	}
	g_free(path);

	gtk_file_chooser_set_preview_widget_active(GTK_FILE_CHOOSER(m_pDlg), 
		!CConfig::GetBool("Environment", "NoPreviews"));

	if (gtk_dialog_run(GTK_DIALOG(m_pDlg))==GTK_RESPONSE_ACCEPT) {
		char *fpath = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(m_pDlg));
		CConfig::SetStr("OpenFile", "File", (fpath ? fpath : ""));
		Global.pMainWnd->ShowFile(fpath);
		g_free(fpath);
	}
}

void COpenDlg::update_preview(GtkFileChooser *widget, COpenDlg *pMe)
{
	pMe->UpdatePreview(widget);
}

void COpenDlg::UpdatePreview(GtkFileChooser *widget)
{
	if (!CConfig::GetBool("Environment", "NoPreviews")) {
		gchar *file = gtk_file_chooser_get_preview_filename(widget);
		if (file) {
			m_Preview.Update(file);
			g_free(file);
		} else {
			m_Preview.Clear();
		}
	}
}

void COpenDlg::response_dialog(GtkDialog *pDlg, gint response_id, COpenDlg *pMe)
{
	if (!pMe->OnCloseQuery(response_id))
		g_signal_stop_emission_by_name(pDlg, "response");
}

bool COpenDlg::OnCloseQuery(gint response_id)
{
	return true;
}

void COpenDlg::selection_changed(GtkFileChooser *widget, COpenDlg *pMe)
{
	pMe->OnSelectionChanged(widget);
}

void COpenDlg::OnSelectionChanged(GtkFileChooser *widget)
{
	UpdateControls();
}

void COpenDlg::UpdateControls(void)
{
	bool ok;

	gchar *fname = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(m_pDlg));
	if (fname) {
		ok = g_file_test(fname, G_FILE_TEST_IS_REGULAR);
		g_free(fname);
	} else {
		ok = false;
	}

	GtkWidget *ok_btn = get_dialog_widget_by_response(m_pDlg, GTK_RESPONSE_ACCEPT);
	gtk_widget_set_sensitive(ok_btn, ok);
}
