/**************************************************************

preview_dlg.cpp (C-Munipack project)
The preview dialog
Copyright (C) 2008 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#include <stdlib.h>
#include <string.h>
#include <glib.h>

#include "utils.h"
#include "project_dlg.h"
#include "main.h"
#include "configuration.h"

enum tLabelId
{
	LABEL_FILENAME,
	LABEL_PATH,
	LABEL_FRAMES,
	LABEL_BIAS,
	LABEL_DARK,
	LABEL_FLAT,
	LABEL_FRAME,
	LABEL_CATALOG,
	LABEL_APERTURE,
	LABEL_VAR,
	LABEL_COMP,
	LABEL_CHK,
	LABEL_OBJNAME,
	LABEL_OBJRA,
	LABEL_OBJDEC,
	LABEL_LOCATION,
	LABEL_LONGITUDE,
	LABEL_LATITUDE
};

//-------------------------   CHOOSE STARS DIALOG   --------------------------------

CProjectDlg::CProjectDlg(GtkWindow *pParent):CInfoDlg(pParent)
{
	// Dialog caption
	gtk_window_set_title(GTK_WINDOW(m_pDlg), "Project info");

	// Dialog icon
	gchar *icon = get_icon_file("project");
	gtk_window_set_icon(GTK_WINDOW(m_pDlg), gdk_pixbuf_new_from_file(icon, NULL));
	g_free(icon);

	AddHeading(0, 0, 0, "Project");
	AddField(LABEL_FILENAME, 0, 1, "File name", PANGO_ELLIPSIZE_MIDDLE);
	AddField(LABEL_PATH, 0, 2, "Path", PANGO_ELLIPSIZE_MIDDLE);
	AddField(LABEL_FRAMES, 0, 3, "No. of frames");
	AddField(LABEL_APERTURE, 0, 4, "Aperture");
	AddSeparator(0, 5);
	AddHeading(0, 0, 6, "Calibration");
	AddField(LABEL_BIAS, 0, 7, "Bias frame", PANGO_ELLIPSIZE_MIDDLE);
	AddField(LABEL_DARK, 0, 8, "Dark frame", PANGO_ELLIPSIZE_MIDDLE);
	AddField(LABEL_FLAT, 0, 9, "Flat frame", PANGO_ELLIPSIZE_MIDDLE);
	AddSeparator(0, 10);
	AddHeading(0, 0, 11, "Reference frame");
	AddField(LABEL_FRAME, 0, 12, "Reference frame");
	AddField(LABEL_CATALOG, 0, 13, "Catalogue file", PANGO_ELLIPSIZE_MIDDLE);
	AddSeparator(0, 14);
	AddHeading(0, 0, 15, "Selected stars");
	AddField(LABEL_VAR, 0, 16, "Variable star(s)");
	AddField(LABEL_COMP, 0, 17, "Comparison star(s)");
	AddField(LABEL_CHK, 0, 18, "Check star(s)");
	AddSeparator(0, 19);
	AddHeading(0, 0, 20, "Object");
	AddField(LABEL_OBJNAME, 0, 21, "Designation");
	AddField(LABEL_OBJRA, 0, 22, "Right ascension");
	AddField(LABEL_OBJDEC, 0, 23, "Declination");
	AddSeparator(0, 24);
	AddHeading(0, 0, 25, "Location");
	AddField(LABEL_LOCATION, 0, 26, "Name");
	AddField(LABEL_LONGITUDE, 0, 27, "Longitude");
	AddField(LABEL_LATITUDE, 0, 28, "Latitude");
	gtk_widget_show_all(m_Tab);
}

void CProjectDlg::ShowModal(void)
{
	int frame;
	char *aux, buf[64];
	double val;
	const char *file;

	SetField(LABEL_FILENAME, Global.pProject->GetFileName());
	SetField(LABEL_PATH, Global.pProject->GetDirPath());
	SetField(LABEL_FRAMES, Global.pProject->GetFileCount());

	SetField(LABEL_BIAS, Global.pProject->GetOrigBiasFile());
	SetField(LABEL_DARK, Global.pProject->GetOrigDarkFile());
	SetField(LABEL_FLAT, Global.pProject->GetOrigFlatFile());

	int aperture = Global.pProject->Aperture();
	if (aperture)
		SetField(LABEL_APERTURE, aperture);
	else
		SetField(LABEL_APERTURE, "------");

	Global.pProject->GetReference(&frame, &file);
	if (frame>0) 
		SetField(LABEL_FRAME, frame);
	else 
		SetField(LABEL_FRAME, "------");
	if (file)
		SetField(LABEL_CATALOG, file);
	else
		SetField(LABEL_CATALOG, "------");

	const CSelection *sel = Global.pProject->Selection();
	aux = sel->GetStarList(ST_VARIABLE);
	SetField(LABEL_VAR, aux);
	g_free(aux);
	aux = sel->GetStarList(ST_COMPARISON);
	SetField(LABEL_COMP, aux);
	g_free(aux);
	aux = sel->GetStarList(ST_CHECK);
	SetField(LABEL_CHK, aux);
	g_free(aux);

	CObjectCoords obj;
	obj = *Global.pProject->ObjectCoords();
	if (obj.Name())
		SetField(LABEL_OBJNAME, obj.Name());
	else
		SetField(LABEL_OBJNAME, "------");
	if (obj.RA() && cmpack_strtora(obj.RA(), &val)==0) {
		cmpack_ratostr(val, buf, 64);
		SetField(LABEL_OBJRA, buf);
	} else
		SetField(LABEL_OBJRA, "------");
	if (obj.Dec() && cmpack_strtodec(obj.Dec(), &val)==0) {
		cmpack_dectostr(val, buf, 64);
		SetField(LABEL_OBJDEC, buf);
	} else 
		SetField(LABEL_OBJDEC, "------");

	CObserverCoords obs;
	if (Global.pProject->UseDefaultLocation())
		obs = Global.pProject->DefaultLocation();
	else
		obs = *Global.pProject->UserLocation();
	if (obs.Name())
		SetField(LABEL_LOCATION, obs.Name());
	else
		SetField(LABEL_LOCATION, "------");
	if (obs.Lon() && cmpack_strtolon(obs.Lon(), &val)==0) {
		cmpack_lontostr(val, buf, 64);
		SetField(LABEL_LONGITUDE, buf);
	} else
		SetField(LABEL_LONGITUDE, "------");
	if (obs.Lat() && cmpack_strtolat(obs.Lat(), &val)==0) {
		cmpack_lattostr(val, buf, 64);
		SetField(LABEL_LATITUDE, buf);
	} else
		SetField(LABEL_LATITUDE, "------");

	CInfoDlg::ShowModal();
}

gboolean CProject::make_status(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, tStatus *st)
{
	int state;

	gtk_tree_model_get(model, iter, FRAME_STATE, &state, -1);
	if (state & CFILE_CONVERSION)
		st->converted++;
	if (state & CFILE_PHOTOMETRY)
		st->photometred++;
	if (state & CFILE_MATCHING)
		st->matched++;
	st->files++;

	return FALSE;
}

void CProject::GetStatus(tStatus *status)
{
	memset(status, 0, sizeof(tStatus));
	gtk_tree_model_foreach(GTK_TREE_MODEL(m_Frames),
		GtkTreeModelForeachFunc(make_status), status);
}
